@@ -296,9 +296,9 @@ void NedBeam::changeStemDir(int dir) {
 void NedBeam::print() {
 	GList *lptr;
 	for (lptr = g_list_first(m_beamed_chords); lptr; lptr = g_list_next(lptr)) {
-		printf("B: 0x%x\n", lptr->data);
+		NedResource::DbgMsg(DBG_TESTING, "B: 0x%x\n", lptr->data);
 	}
-	printf("############\n");
+	NedResource::DbgMsg(DBG_TESTING, "############\n");
 }
 
 double NedBeam::getYPosAt(double xpos) {
@@ -319,6 +319,10 @@ void NedBeam::draw(cairo_t *cr) {
 		drawGraceBeam(cr);
 		return;
 	}
+	if (getMainWindow()->doDrawPostscript()) {
+		drawPostscript(cr);
+		return;
+	}
 	int dir;
 	double topy = getMainWindow()->getTopY();
 	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
@@ -424,6 +428,119 @@ void NedBeam::drawGraceBeam(cairo_t *cr) {
 	cairo_stroke(cr);
 }
 
+void NedBeam::drawPostscript(cairo_t *cr) {
+	int dir;
+	double topy = getMainWindow()->getTopY();
+	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
+	double leftx = getMainWindow()->getLeftX();
+	GList *lptr1, *lptr2, *last;
+	NedChordOrRest *chord1, *chord2, *lastchord;
+	double xpos1, xpos2, ypos1, ypos2;
+	double mid_xpos, mid_ypos;
+	double lastxpos, lastypos;
+	int tval;
+	cairo_glyph_t glyph;
+	double h_2 = BEAM_THICK / 2.0;
+
+
+
+	cairo_new_path(cr);
+	cairo_set_line_width(cr, 0.1);
+	//cairo_set_line_width(cr, zoom_factor * BEAM_THICK);
+	dir = m_up ? 1 : -1;
+	lptr1 = g_list_first(m_beamed_chords);
+	last = g_list_last(m_beamed_chords);
+	chord1 = (NedChordOrRest *) lptr1->data;
+	lastchord = (NedChordOrRest *) last->data;
+	xpos1 = chord1->getBeamXPos() - STEM_THICK / 2.0;
+	ypos1 = m_start_y - (xpos1 - m_start_x) * m_slope;
+	lastxpos = lastchord->getBeamXPos() + STEM_THICK / 2.0;
+	lastypos = m_start_y - (lastxpos - m_start_x) * m_slope;
+	cairo_move_to(cr, X_POS_PAGE_REL(xpos1), Y_POS_STAFF_REL(ypos1 - h_2));
+	cairo_line_to(cr, X_POS_PAGE_REL(lastxpos), Y_POS_STAFF_REL(lastypos - h_2));
+	cairo_line_to(cr, X_POS_PAGE_REL(lastxpos), Y_POS_STAFF_REL(lastypos + h_2));
+	cairo_line_to(cr, X_POS_PAGE_REL(xpos1), Y_POS_STAFF_REL(ypos1 + h_2));
+	cairo_close_path(cr);
+	cairo_fill(cr);
+	for (lptr2 = g_list_next(lptr1); lptr2; lptr1 = lptr2, lptr2 = g_list_next(lptr2), chord1 = chord2, xpos1 = xpos2, ypos1 = ypos2) {
+		chord2 = (NedChordOrRest *) lptr2->data;
+		xpos2 = m_up ? chord2->getBeamXPos() : chord2->getBeamXPos();
+		ypos2 = m_start_y - (xpos2 - m_start_x) * m_slope;
+		mid_xpos = xpos1 + (xpos2 - xpos1) / 2.0;
+		mid_ypos = ypos1 + (ypos2 - ypos1) / 2.0;
+		switch (chord1->getLength()) {
+		  case NOTE_64:
+			cairo_new_path(cr);
+			cairo_move_to(cr, X_POS_PAGE_REL(xpos1), Y_POS_STAFF_REL(ypos1 + 3 * dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + 3 * dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + 3 * dir * FLAG_DIST + h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos1), Y_POS_STAFF_REL(ypos1 + 3 * dir * FLAG_DIST + h_2));
+			cairo_close_path(cr);
+			cairo_fill(cr);
+		  case NOTE_32:
+			cairo_new_path(cr);
+			cairo_move_to(cr, X_POS_PAGE_REL(xpos1), Y_POS_STAFF_REL(ypos1 + 2 * dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + 2 * dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + 2 * dir * FLAG_DIST + h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos1), Y_POS_STAFF_REL(ypos1 + 2 * dir * FLAG_DIST + h_2));
+			cairo_close_path(cr);
+			cairo_fill(cr);
+		  case NOTE_16:
+			cairo_new_path(cr);
+			cairo_move_to(cr, X_POS_PAGE_REL(xpos1), Y_POS_STAFF_REL(ypos1 + dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + dir * FLAG_DIST + h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos1), Y_POS_STAFF_REL(ypos1 + dir * FLAG_DIST + h_2));
+			cairo_close_path(cr);
+			cairo_fill(cr);
+		}
+		if (lptr2 == last || chord2->getLength() >= chord1->getLength()) {
+		   switch (chord2->getLength()) {
+		     case NOTE_64:
+			cairo_new_path(cr);
+			cairo_move_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + 3 * dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos2), Y_POS_STAFF_REL(ypos2 + 3 * dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos2), Y_POS_STAFF_REL(ypos2 + 3 * dir * FLAG_DIST + h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + 3 * dir * FLAG_DIST + h_2));
+			cairo_close_path(cr);
+			cairo_fill(cr);
+		     case NOTE_32:
+			cairo_new_path(cr);
+			cairo_move_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + 2 * dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos2), Y_POS_STAFF_REL(ypos2 + 2 * dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos2), Y_POS_STAFF_REL(ypos2 + 2 * dir * FLAG_DIST + h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + 2 * dir * FLAG_DIST + h_2));
+			cairo_close_path(cr);
+			cairo_fill(cr);
+		     case NOTE_16:
+			cairo_new_path(cr);
+			cairo_move_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos2), Y_POS_STAFF_REL(ypos2 + dir * FLAG_DIST - h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(xpos2), Y_POS_STAFF_REL(ypos2 + dir * FLAG_DIST + h_2));
+			cairo_line_to(cr, X_POS_PAGE_REL(mid_xpos), Y_POS_STAFF_REL(mid_ypos + dir * FLAG_DIST + h_2));
+			cairo_close_path(cr);
+			cairo_fill(cr);
+		   }
+		}
+	}
+	cairo_stroke(cr);
+	cairo_new_path(cr);
+	if (m_tuplet_marker > 0) {
+		tval = m_tuplet_marker;
+		glyph.y = Y_POS_STAFF_REL(m_tuplet_y + (m_up ? -(NUMNBER_GLYPH_HEIGHT_UP + TUPLET_MARKER_DIST) : (NUMNBER_GLYPH_HEIGHT_DOWN + TUPLET_MARKER_DIST)));
+		if (m_tuplet_marker > 9) {
+			glyph.index = NUMBERBASE + 1;
+			glyph.x = X_POS_PAGE_REL(m_tuplet_x - FIRST_TUPLET_DIGIT_DIST);
+			cairo_show_glyphs(cr, &glyph, 1);
+			tval -= 10;
+		}
+		glyph.index = NUMBERBASE + tval;
+		glyph.x = X_POS_PAGE_REL(m_tuplet_x);
+		cairo_show_glyphs(cr, &glyph, 1);
+	}
+	cairo_stroke(cr);
+}
+
 void NedBeam::saveBeam(FILE *fp) {
 	GList *lptr;
 	int marker;
