@@ -37,12 +37,12 @@ _("F sharp Major; d sharp minor")
 };
 
 
-NedStaffContextDialog::NedStaffContextDialog(GtkWindow *parent, int staff_count, int staff_pos, char *staff_name, int clef_number, int octave_shift, int key_signature_number, int numerator, int demoninator, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int reverb) :
+NedStaffContextDialog::NedStaffContextDialog(GtkWindow *parent, int staff_count, int staff_pos, char *staff_name, int clef_number, int octave_shift, int key_signature_number, int numerator, int demoninator, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int reverb, bool muted) :
 m_octave_shift(octave_shift), m_tempo(tempo), m_pan(pan), m_chorus(chorus), m_reverb(reverb), m_volume(volume), m_channel_value(channel),
 m_state(false), m_numerator_value(numerator), m_denominator_value(demoninator),
 m_key_signature_number(key_signature_number), m_current_clef(clef_number), m_midi_program_number(midi_program), m_config_changed(false),
 m_delete_staff(false), m_staff_pos(staff_pos), 
-m_staff_count(staff_count), m_staff_name(staff_name) {
+m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	GtkWidget *dialog;
 	GtkWidget *label_score;
 	GtkWidget *label_midi;
@@ -77,6 +77,8 @@ m_staff_count(staff_count), m_staff_name(staff_name) {
 	GtkTreeViewColumn *instrument_column;
 	GtkTreeViewColumn *instrument_num_column;
 	GtkWidget *channel_frame;
+	GtkWidget *mute_frame;
+	GtkWidget *mute_channel_vbox;
 	GdkColor bgcolor;
 	GtkTreePath* instrument_tree_path;
 	GtkListStore *position_list_store;
@@ -256,6 +258,15 @@ m_staff_count(staff_count), m_staff_name(staff_name) {
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_channel_spin_box), m_channel_value + 1);
 	gtk_container_add (GTK_CONTAINER(channel_frame), m_channel_spin_box);
 
+	mute_frame = gtk_frame_new(_("muting"));
+	m_mute_box = gtk_check_button_new();
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_mute_box), m_muted);
+	gtk_container_add (GTK_CONTAINER(mute_frame), m_mute_box);
+
+	mute_channel_vbox = gtk_vbox_new(FALSE, 2);
+	gtk_box_pack_start(GTK_BOX(mute_channel_vbox), channel_frame,  FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(mute_channel_vbox), mute_frame, FALSE, FALSE, 0);
+
 	if (staff_count > 1) {
 
 		position_list_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
@@ -305,7 +316,7 @@ m_staff_count(staff_count), m_staff_name(staff_name) {
 	gtk_box_pack_start(GTK_BOX(midi_hbox), reverb_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), m_instrument_list, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), instrument_scroll, FALSE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(midi_hbox), channel_frame, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(midi_hbox), mute_channel_vbox, FALSE, TRUE, 0);
 
 
 	label_score = gtk_label_new(_("Score"));
@@ -330,6 +341,7 @@ m_staff_count(staff_count), m_staff_name(staff_name) {
 
 void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer data) {
 	int i;
+	bool newmute;
 	GtkTreeSelection* selection;
 	GList *selected_rows;
 	int newchannel, newvolume, newpgm;
@@ -401,6 +413,11 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 		context_dialog->m_reverb = newreverb;
 		context_dialog->m_config_changed = TRUE;
 	}
+	newmute = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_mute_box));
+	if (newmute != context_dialog->m_muted) {
+		context_dialog->m_muted = newmute;
+		context_dialog->m_config_changed = TRUE;
+	}
 	newname = gtk_entry_get_text(GTK_ENTRY(context_dialog->m_staffname));
 	bool diff = (newname == NULL && context_dialog->m_staff_name != NULL) || 
 		(newname != NULL && context_dialog->m_staff_name == NULL);
@@ -463,7 +480,7 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 	gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 
-void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, int *staff_pos, char **staff_name, int *clef_number, int *octave_shift, int *key_signature_number, int *numerator, int *demoninator, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *reverb, bool *config_changed) {
+void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, int *staff_pos, char **staff_name, int *clef_number, int *octave_shift, int *key_signature_number, int *numerator, int *demoninator, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *reverb, bool *config_changed, bool *muted) {
 	*delete_staff = m_delete_staff;
 	*state = m_state;
 	*staff_pos = m_staff_pos;
@@ -484,6 +501,7 @@ void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, int *staf
 	*octave_shift = m_octave_shift;
 	*midi_program = m_midi_program_number;
 	*channel = m_channel_value;
+	*muted = m_muted;
 	if (m_state) {
 		*config_changed = m_config_changed;
 	}
