@@ -36,6 +36,9 @@ class NedCommandList;
 class NedChordOrRest;
 class NedSystem;
 class NedStaff;
+class NedFreeReplaceable;
+class NedFreeText;
+class NedImporter;
 class NedMusicXMLImport;
 class ScoreInfo;
 struct paper_info_struct;
@@ -45,12 +48,13 @@ struct staff_context_str {
 	int m_key_signature_number;
 	int m_clef_number;
 	int m_clef_octave_shift;
-	int m_midi_volume;
-	int m_midi_pan;
-	int m_midi_chorus;
-	int m_midi_reverb;
-	int m_midi_program;
-	int m_midi_channel;
+	unsigned char m_midi_volume;
+	unsigned char m_current_midi_volume;
+	unsigned char m_midi_pan;
+	unsigned char m_midi_chorus;
+	unsigned char m_midi_reverb;
+	unsigned char m_midi_program;
+	unsigned char m_midi_channel;
 	unsigned int m_flags;
 #define BRACE_START   (1 << 0)
 #define BRACE_END     (1 << 1)
@@ -58,6 +62,12 @@ struct staff_context_str {
 #define BRACKET_END   (1 << 3)
 #define CON_BAR_START (1 << 4)
 #define CON_BAR_END   (1 << 5)
+	unsigned long long m_crescendo_midi_start;
+	unsigned long long m_crescendo_midi_end;
+	unsigned char m_crescendo_start_volume;
+	unsigned char m_crescendo_end_volume;
+	int m_pitch_offs;
+	bool m_muted;
 
 };
 
@@ -121,10 +131,12 @@ class NedMainWindow {
 		double getTopY();
 		NedNote *m_selected_note;
 		NedChordOrRest *m_selected_chord_or_rest;
+		NedFreeReplaceable *m_selected_free_replaceable;
 		SpecialMeasure *m_selected_spec_measure;
 		GdkCursor *m_hand, *m_pointer, *m_pencil;
 		NedSystem *getNextSystem(NedPage *page, NedCommandList *command_list = NULL);
 		NedPage *getNextPage(NedPage *page, NedCommandList *command_list = NULL);
+		NedPage *getPreviousPage(NedPage *page);
 		int getSorting(NedPage *this_page, NedStaff *this_staff, NedPage *other_page, NedStaff *other_staff);
 #ifdef WITH_TIME_TEST
 		struct timeval m_last_mouse_motion_time;
@@ -132,11 +144,12 @@ class NedMainWindow {
 		void setUnRedoButtons(bool execute_possible, bool unexecute_possible);
 		void repaint();
 		void repaintDuringReplay(NedNote *notes[], int num_notes);
-		void reposit(NedCommandList *command_list = NULL, NedPage *start_page = NULL, NedSystem *start_system = NULL, bool full = false);
+		void reposit(NedCommandList *command_list = NULL, NedPage *start_page = NULL, NedSystem *start_system = NULL, bool full = false,  bool with_progress_bar = false);
 		void renumberMeasures(NedSystem *start_system = NULL, int measure_number = 1, bool force = false);
 		staff_context_str m_staff_contexts[MAX_STAFFS];
 		int getNumerator() {return m_numerator;}
 		int getDenominator() {return m_denominator;}
+		bool isTimsigChangingMeasure(int meas_num);
 		unsigned int getMeasureDuration(int meas_num);
 		bool findTimeOfMeasure(int meas_num, unsigned long long *meas_time);
 		int getNumerator(int meas_num);
@@ -147,10 +160,7 @@ class NedMainWindow {
 		void setVisiblePage(NedPage *page);
 		void setVisible(NedNote *note);
 		void setVisible(NedChordOrRest *chor_or_rest);
-		void setSelected(NedChordOrRest *chord_or_rest, NedNote *note) {
-			m_selected_chord_or_rest = chord_or_rest;
-			m_selected_note = note;
-		}
+		void setSelected(NedChordOrRest *chord_or_rest, NedNote *note);
 		NedPage *getLastPage();
 		ScoreInfo *getScoreInfo() {return m_score_info;}
 		double getFirstPageYOffs() {return m_first_page_yoffs;}
@@ -159,11 +169,15 @@ class NedMainWindow {
 		void renumberPages();
 		int getStaffCount() {return m_staff_count;}
 		void do_remove_last_staff();
-		bool do_musicxml_import(NedMusicXMLImport *im);
+		bool do_import_from_other(NedImporter *im);
+		void do_lily_export(FILE *fp);
 		void deleteStaff(int staff_number);
 		void shiftStaff(int staff_number, int position);
 		void restoreStaff(int staff_number, staff_context_str *staff_context);
+		unsigned int determineVolume(NedChordOrRest *element, int staff_nr);
+		double determineTempoInverse(NedChordOrRest *element);
 		GtkWidget *getDrawingArea() {return m_drawing_area;}
+		GtkWidget *getWindow() {return m_main_window;}
 		struct paper_info_struct *getCurrentPaper() {return m_current_paper;}
 		bool getPortrait() {return m_portrait;}
 		void reconfig_paper(bool do_reposit, struct paper_info_struct *paper_type, bool portrait);
@@ -177,11 +191,14 @@ class NedMainWindow {
 		bool needsARepLine(int measure_number);
 		int getSpecialType() {return m_special_type;}
 		int getSpecialSubType() {return m_special_sub_type;}
+		unsigned int getSpecial(int meas_num);
+		SpecialMeasure *getSpecialMeasure(int meas_num);
 		void resetSpecialType() {m_special_type = m_special_sub_type = -1;}
 		void setAndUpdateClefTypeAndKeySig();
 		double getFirstSystemIndent() {return m_first_system_indent;}
 		unsigned int getUpBeatInverse() {return m_upbeat_inverse;}
 		int getFileVersion() {return m_file_version;}
+		NedFreeReplaceable *m_freetext;
 	private: 
 		GList *m_special_measures;
 		int setSpecial(int measnum, int mtype);
@@ -191,6 +208,9 @@ class NedMainWindow {
 		bool conflictWithOhterSpecMeasure(int measnum, int dir);
 		void testAlternativeState(NedCommandList *command_list, NedMeasure *measure, bool alter1, bool alter2);
 		void checkForElementsToSplit(NedCommandList *command_list);
+		void import_from_other(NedImporter *im, const char *filter_name, const char *pat1, const char *pat2, const char *label, char **last_dir);
+		void deleteEmptyPagesAtEnd();
+		void find_new_cursor_pos(guint keyval, int *x, int *y);
 		void disconnectKeys();
 		bool getTiedMode();
 		void resetSomeButtons();
@@ -202,6 +222,8 @@ class NedMainWindow {
 		unsigned int getMainWindowStatus();
 		void modeChange(unsigned int stat, GtkAction *action);;
 		char *m_start_file_name;
+		int m_last_line_during_key_insertion;
+		NedStaff *m_last_staff_during_key_insertion;
 		double m_current_zoom;
 		double m_leftx, m_topy;
 		double m_leftx0, m_topy0;
@@ -238,14 +260,18 @@ class NedMainWindow {
 		GtkWidget *m_more_acc_bar;
 		GtkAction *m_undo_action, *m_redo_action;
 		GtkToggleAction *m_sharp_action, *m_flat_action, *m_natural_action, *m_insert_erease_mode_action;
-		GtkToggleAction *m_stacc_action;
+		GtkToggleAction *m_stacc_action, *m_staccatissimo_action;
 		GtkToggleAction *m_tenuto_action;
 		GtkToggleAction *m_sforzato_action;
 		GtkToggleAction *m_sforzando_action;
 		GtkToggleAction *m_bow_up_action;
 		GtkToggleAction *m_bow_down_action;
+		GtkToggleAction *m_ped_on_action;
+		GtkToggleAction *m_ped_off_action;
+		GtkToggleAction *m_trill_action, *m_prall_action, *m_open_action, *m_mordent_action;
+		GtkToggleAction *m_arpeggio_action, *m_fermata_action;
 		GtkToggleAction *m_dsharp_action, *m_dflat_action;
-		GtkToggleAction *m_shift_action;
+		GtkToggleAction *m_shift_action, *m_keyboard_insert_action;
 		GtkToggleAction *m_pause_action, *m_dotted_action, *m_ddotted_action;
 		GtkToggleAction *m_tie_action;
 		GtkToggleAction *m_replay_action;
@@ -302,7 +328,9 @@ class NedMainWindow {
 		static void restore_score(GtkWidget *widget, void *data);
 		static void print_file(GtkWidget *widget, void *data);
 		static void import_musicxml(GtkWidget *widget, void *data);
+		static void import_midi(GtkWidget *widget, void *data);
 		static void export_midi(GtkWidget *widget, void *data);
+		static void export_lily(GtkWidget *widget, void *data);
 		static void insert_tuplet(GtkWidget *widget, void *data);
 		static void set_brace_system_delimiter(GtkWidget *widget, void *data);
 		static void set_bracket_system_delimiter(GtkWidget *widget, void *data);
@@ -312,16 +340,35 @@ class NedMainWindow {
 		static void append_page(GtkWidget *widget, void *data);
 		static void insert_page(GtkWidget *widget, void *data);
 		static void insert_keysig(GtkWidget *widget, void *data);
+		static void insert_lines(GtkWidget *widget, void *data);
+		static void insert_signs(GtkWidget *widget, void *data);
+		static void insert_slur(GtkWidget *widget, void *data);
+		static void insert_accelerato(GtkWidget *widget, void *data);
+		static void insert_ritardando(GtkWidget *widget, void *data);
 		static void append_staff(GtkWidget *widget, void *data);
+		static void insert_ppp(GtkAction *widget, gpointer data);
+		static void insert_pp(GtkAction *widget, gpointer data);
+		static void insert_p(GtkAction *widget, gpointer data);
+		static void insert_mp(GtkAction *widget, gpointer data);
+		static void insert_sp(GtkAction *widget, gpointer data);
+		static void insert_mf(GtkAction *widget, gpointer data);
+		static void insert_sf(GtkAction *widget, gpointer data);
+		static void insert_f(GtkAction *widget, gpointer data);
+		static void insert_ff(GtkAction *widget, gpointer data);
+		static void insert_fff(GtkAction *widget, gpointer data);
+		static void insert_tempo(GtkAction *widget, gpointer data);
+		static void insert_text(GtkAction *widget, gpointer data);
 		static void set_lyrics_mode1(GtkAction *widget, gpointer data);
 		static void set_lyrics_mode2(GtkAction *widget, gpointer data);
 		static void set_lyrics_mode3(GtkAction *widget, gpointer data);
 		static void set_lyrics_mode4(GtkAction *widget, gpointer data);
 		static void set_lyrics_mode5(GtkAction *widget, gpointer data);
 		static void remove_page(GtkWidget *widget, void *data);
+		static void remove_empty_pages(GtkWidget *widget, void *data);
 		static void set_upbeat_measure(GtkWidget *widget, void *data);
 		static void set_upbeat_start(GtkWidget *widget, void *data);
 		static void edit_score_info(GtkWidget *widget, void *data);
+		static void mute_staves(GtkWidget *widget, void *data);
 		static void config_midi(GtkWidget *widget, void *data);
 		static void config_paper(GtkWidget *widget, void *data);
 		static void show_about(GtkWidget *widget, void *data);
@@ -349,12 +396,21 @@ class NedMainWindow {
 		static void set_dcross_mode(GtkAction *action, gpointer data);
 		static void set_dflat_mode(GtkAction *action, gpointer data);
 		static void set_staccato_mode(GtkAction *action, gpointer data);
+		static void set_staccatissimo_mode(GtkAction *action, gpointer data);
 //		static void set_str_pizz_mode(GtkAction *action, gpointer data);
 		static void set_tenuto_mode(GtkAction *action, gpointer data);
 		static void set_sforzato_mode(GtkAction *action, gpointer data);
 		static void set_sforzando_mode(GtkAction *action, gpointer data);
 		static void set_bow_up_mode(GtkAction *action, gpointer data);
 		static void set_bow_down_mode(GtkAction *action, gpointer data);
+		static void set_pedal_on_mode(GtkAction *action, gpointer data);
+		static void set_pedal_off_mode(GtkAction *action, gpointer data);
+		static void set_trill_mode(GtkAction *action, gpointer data);
+		static void set_prall_mode(GtkAction *action, gpointer data);
+		static void set_open_mode(GtkAction *action, gpointer data);
+		static void set_mordent_mode(GtkAction *action, gpointer data);
+		static void set_fermata_mode(GtkAction *action, gpointer data);
+		static void set_arpeggio_mode(GtkAction *action, gpointer data);
 		static void set_tied_mode(GtkAction *action, gpointer data);
 		static void do_play(GtkAction *action, gpointer data);
 		static void set_colored(GtkAction *action, gpointer data);
@@ -423,6 +479,7 @@ class NedMainWindow {
 	friend class NedMidiExport;
 	friend class NedMusicXMLImport;
 	friend class NedChangeUpbeadCommand;
+	friend class NedInstrumentTable;
 };
 
 #endif /* MAINWINDOW_H */
