@@ -74,74 +74,8 @@
 #define XML_TIE_TYPE_START 1
 #define XML_TIE_TYPE_STOP 2
 
-part::part() {
-	init();
-}
-void part::init() {
-	int i;
-	id[0] = '\0';
-	name[0] = '\0';
-	divisions = 1;
-	current_clef = clef = TREBLE_CLEF;
-	clef_octave = current_clef_octave = 0;
-	first_clef = true;
-	current_key_signature = key_signature = 0;
-	first_key_signature = true;
-	numerator = current_numerator = denominator = current_denominator = 4;
-	first_time_signature = true;
-	midi_pgm = 0;
-	midi_channel = 0;
-	for (i = 0; i < VOICE_COUNT; i++) {
- 		voices[i] = NULL;
-		start_of_last_taken[i] = NULL;
-		end_of_last_taken[i] = NULL;
-		last_chord[i] = NULL;
-		eof[i] = false;
-		last_keysig[i] = 0;
-	}
-}
-
-bool part::is_eof() {
-	int i;
-	for (i = 0; i < VOICE_COUNT; i++) {
-		if (!eof[i]) return false;
-	}
-	return true;
-}
-
-bool part::isEmpty() {
-	int i;
-	for (i = 0; i < VOICE_COUNT; i++) {
-		if (g_list_length(voices[i]) > 0 ) return false;
-	}
-	return true;
-}
-
-void part::copy_from(part *other_part) {
-	int i;
-	*this = *other_part;
-	strcpy(id, other_part->id);
-	strcpy(name, other_part->name);
-	for (i = 0; i < VOICE_COUNT; i++) {
-		voices[i] = other_part->voices[i];
-		start_of_last_taken[i] = other_part->start_of_last_taken[i];
-		end_of_last_taken[i] = other_part->end_of_last_taken[i];
-		eof[i] = other_part->eof[i];
-	}
-}
-
-void part::resetTaken() {
-	int i;
-
-	for (i = 0; i < VOICE_COUNT; i++) {
-		end_of_last_taken[i] = g_list_previous(start_of_last_taken[i]);
-	}
-}
 		
 
-
-NedMusicXMLImport::NedMusicXMLImport(NedMainWindow *mainwindow, FILE *fp) : m_main_window(mainwindow), m_FILE(fp) {}
-
 bool NedMusicXMLImport::import() {
 #define BUFLEN 4096
 	GMarkupParser parser;
@@ -188,19 +122,19 @@ bool NedMusicXMLImport::import() {
 	g_markup_parse_context_free(m_parser_context);
 	/*
 	for (int i = 0; i < m_partcount; i++) {
-		printf("Part %d: id = %s, name = %s, divisions = %d, midi_pgm = %d, midi_channel = %d clef = ",
+		NedResource::DbgMsg(DBG_TESTING, "Part %d: id = %s, name = %s, divisions = %d, midi_pgm = %d, midi_channel = %d clef = ",
 		 i, m_parts[i].id, m_parts[i].name, m_parts[i].divisions,
 		m_parts[i].midi_pgm, m_parts[i].midi_channel);
 		switch(m_parts[i].clef) {
-			case TREBLE_CLEF: printf("TREBLE_CLEF\n"); break; 
-			case ALTO_CLEF: printf("ALTO_CLEF\n"); break; 
-			case BASS_CLEF: printf("BASS_CLEF\n"); break; 
-			case NEUTRAL_CLEF1: printf("NEUTRAL_CLEF1\n"); break;
-			case NEUTRAL_CLEF2: printf("NEUTRAL_CLEF2\n"); break;
+			case TREBLE_CLEF: NedResource::DbgMsg(DBG_TESTING, "TREBLE_CLEF\n"); break; 
+			case ALTO_CLEF: NedResource::DbgMsg(DBG_TESTING, "ALTO_CLEF\n"); break; 
+			case BASS_CLEF: NedResource::DbgMsg(DBG_TESTING, "BASS_CLEF\n"); break; 
+			case NEUTRAL_CLEF1: NedResource::DbgMsg(DBG_TESTING, "NEUTRAL_CLEF1\n"); break;
+			case NEUTRAL_CLEF2: NedResource::DbgMsg(DBG_TESTING, "NEUTRAL_CLEF2\n"); break;
 		}
-		printf("num = %d, denom = %d, keysig = %d\n",
+		NedResource::DbgMsg(DBG_TESTING, "num = %d, denom = %d, keysig = %d\n",
 			 m_parts[i].numerator, m_parts[i].denominator, m_parts[i].key_signature);
-		fflush(stdout);
+		
 	}
 	*/
 	if (m_upbeat_inverse ==  1) m_upbeat_inverse = 0; 
@@ -208,14 +142,6 @@ bool NedMusicXMLImport::import() {
 	return !m_semantic_error;
 }
 
-bool NedMusicXMLImport::is_eof() {
-	int i;
-
-	for (i = 0; i < m_partcount; i++) {
-		if (!m_parts[i].is_eof()) return false;
-	}
-	return true;
-}
 
 void NedMusicXMLImport::produceSyntaxErrorString() {
 	if (!m_semantic_error) {
@@ -248,7 +174,7 @@ int NedMusicXMLImport::pitchToLine() {
 		produceSementicErrorString(Str);
 		return UNKNOWN_LINE;
 	}
-	if (m_octave_name[1] != '\0' || m_octave_name[0] < '1' || m_octave_name[0] > '7') {
+	if (m_octave_name[1] != '\0' || m_octave_name[0] < '0' || m_octave_name[0] > '7') {
 		sprintf(Str, _("unknown octave: %s"), m_octave_name);
 		produceSementicErrorString(Str);
 		return UNKNOWN_LINE;
@@ -286,15 +212,15 @@ void NedMusicXMLImport::startTag (GMarkupParseContext *context,
 /*
 	int i, j;
 	for (i = 0; i < im->m_indent; i++) putchar(' ');
-	printf("Start: %s\n", element_name);
+	NedResource::DbgMsg(DBG_TESTING, "Start: %s\n", element_name);
 	j = 0;
 	while (attribute_names[j] != NULL) {
-		for (i = 0; i < im->m_indent+1; i++) putchar(' ');
-		printf("[ %s : %s ]\n", attribute_names[j], attribute_values[j]);
+		for (i = 0; i < im->m_indent+1; i++) NedResource::DbgMsg(DBG_TESTING, " ");
+		NedResource::DbgMsg(DBG_TESTING, "[ %s : %s ]\n", attribute_names[j], attribute_values[j]);
 		j++;
 	}
 
-	fflush(stdout);
+	
 */
 	if (!strcmp(element_name, "note")) {
 		im->push_state(XML_STATE_NOTE);
@@ -305,6 +231,11 @@ void NedMusicXMLImport::startTag (GMarkupParseContext *context,
 		im->m_is_grace = false;
 		im->m_has_splash = false;
 		im->m_staccato = false;
+		im->m_staccatissimo = false;
+		im->m_trill = false;
+		im->m_strong_accent = false;
+		im->m_tenuto = false;
+		im->m_accent = false;
 		im->m_note_type[0] = '\0';
 		im->m_is_chord_member = false;
 		im->m_alter = UNKNOWN_LINE;
@@ -340,6 +271,54 @@ void NedMusicXMLImport::startTag (GMarkupParseContext *context,
 		}
 		im->m_staccato = true;
 	}
+	else if (!strcmp(element_name, "staccatissimo")) {
+		if (!im->check_state(XML_STATE_ARTICULATIONS, XML_STATE_NOTE)) {
+			im->produceSementicErrorString(_("\"articulations\" outside \"note\""));
+			return;
+		}
+		im->m_staccatissimo = true;
+	}
+	else if (!strcmp(element_name, "strong-accent")) {
+		if (!im->check_state(XML_STATE_ARTICULATIONS, XML_STATE_NOTE)) {
+			im->produceSementicErrorString(_("\"strong-accent\" outside \"note\""));
+			return;
+		}
+		im->m_strong_accent = true;
+	}
+	else if (!strcmp(element_name, "tenuto")) {
+		if (!im->check_state(XML_STATE_ARTICULATIONS, XML_STATE_NOTE)) {
+			im->produceSementicErrorString(_("\"tenuto\" outside \"note\""));
+			return;
+		}
+		im->m_tenuto = true;
+	}
+	else if (!strcmp(element_name, "accent")) {
+		if (!im->check_state(XML_STATE_ARTICULATIONS, XML_STATE_NOTE)) {
+			im->produceSementicErrorString(_("\"accent\" outside \"note\""));
+			return;
+		}
+		im->m_accent = true;
+	}
+	else if (!strcmp(element_name, "trill-mark")) {
+		im->m_trill = true;
+	}
+	else if (!strcmp(element_name, "pedal")) {
+		if (!im->getAttr("type", buffer, attribute_names, attribute_values)) {
+			im->produceSementicErrorString(_("\"pedal\" without \"type\""));
+			return;
+		}
+		if (!strcmp(buffer, "start")) {
+			im->m_pedal_on = true;
+		}
+		else if (!strcmp(buffer, "stop")) {
+			im->m_pedal_off = true;
+		}
+		else {
+			im->produceSementicErrorString(_("\"pedal\" --> \"type\" is bad"));
+			return;
+		}
+			
+	}
 	else if (!strcmp(element_name, "alter")) {
 		im->push_state(XML_STATE_ALTER);
 	}
@@ -383,6 +362,8 @@ void NedMusicXMLImport::startTag (GMarkupParseContext *context,
 			im->produceSementicErrorString(Str);
 			return;
 		}
+		im->m_pedal_off = false;
+		im->m_pedal_on = false;
 		
 	}
 	else if (!strcmp(element_name, "midi-instrument")) {
@@ -562,14 +543,14 @@ void NedMusicXMLImport::endTag (GMarkupParseContext *context,
 	int i;
 	im->m_indent -= INC_INDENT;
 	for (i = 0; i < im->m_indent; i++) putchar(' ');
-	printf("Ende: %s\n", element_name); fflush(stdout);
+	NedResource::DbgMsg(DBG_TESTING, "Ende: %s\n", element_name); 
 */
 	if (!strcmp(element_name, "note")) {
 		im->pop_state();
 		if (im->m_current_part - im->m_parts >= MAXPARTS) return;
 		if (im->m_current_voice < 0 || im->m_current_voice >= VOICE_COUNT) {
 			//sprintf(Str, _("voice number: %d is not supported"), im->m_current_voice + 1);
-			fprintf(stderr, _("voice number: %d is not supported"), im->m_current_voice + 1);
+			NedResource::DbgMsg(DBG_CRITICAL, _("voice number: %d is not supported"), im->m_current_voice + 1);
 			//im->produceSementicErrorString(Str);
 			return;
 		}
@@ -585,16 +566,16 @@ void NedMusicXMLImport::endTag (GMarkupParseContext *context,
 		im->determineLength(&length, &dotcount);
 /*
 		if (im->m_actual_notes > 0) {
-			printf("TOL: %d/%d, ", im->m_actual_notes, im->m_normal_notes);
+			NedResource::DbgMsg(DBG_TESTING, "TOL: %d/%d, ", im->m_actual_notes, im->m_normal_notes);
 		}
 		if (im->m_is_rest) {
-			printf("Rest: length = %u(%u), dotcount = %d, voice = %d\n", length, length/NOTE_8, dotcount, im->m_current_voice);
+			NedResource::DbgMsg(DBG_TESTING, "Rest: length = %u(%u), dotcount = %d, voice = %d\n", length, length/NOTE_8, dotcount, im->m_current_voice);
 		}
 		else if ((line = im->pitchToLine()) != UNKNOWN_LINE) {
-			printf("length = %u(%u), dotcount = %d, line = %d, chord_member = %d, voice = %d\n",
+			NedResource::DbgMsg(DBG_TESTING, "length = %u(%u), dotcount = %d, line = %d, chord_member = %d, voice = %d\n",
 				 length, length/NOTE_8, dotcount, line, im->m_is_chord_member, im->m_current_voice);
 		}
-		fflush(stdout);
+		
 */
 		if (im->m_is_chord_member) {
 			if (im->m_current_part->last_chord[im->m_current_voice] == NULL) {
@@ -724,7 +705,7 @@ void NedMusicXMLImport::internal_text(GMarkupParseContext *context,
 	char Str[DEFAULT_STRING_LENGTH];
 /*
 	int i;
-	printf("TEXT(0): %s, state = %d\n", text, im->getState()); fflush(stdout);
+	NedResource::DbgMsg(DBG_TESTING, "TEXT(0): %s, state = %d\n", text, im->getState()); 
 	char *cptr1, *cptr2;
 	if (text_len > 0) return;
 	for (cptr1 = (char *) text; *cptr1 <= ' ' && *cptr1 != '\0'; cptr1++);
@@ -732,8 +713,8 @@ void NedMusicXMLImport::internal_text(GMarkupParseContext *context,
 	cptr2[1] = '\0';
 	if (strlen(cptr1) < 1) return;
 	for (i = 0; i < im->m_indent; i++) putchar(' ');
-	printf("TEXT: %s\n", cptr1); fflush(stdout);
-	printf("TEXT: %s, state = %d\n", cptr1, im->getState()); fflush(stdout);
+	NedResource::DbgMsg(DBG_TESTING, "TEXT: %s\n", cptr1); 
+	NedResource::DbgMsg(DBG_TESTING, "TEXT: %s, state = %d\n", cptr1, im->getState()); 
 */
 	switch(im->getState()) {
 		case XML_STATE_STEP: if (im->m_pitch_name[0] != '\0') {
@@ -750,7 +731,7 @@ void NedMusicXMLImport::internal_text(GMarkupParseContext *context,
 				     break;
 		case XML_STATE_DURATION: 
 				     if (im->m_duration >= 0) {
-					printf("text = %s\n", text);
+					NedResource::DbgMsg(DBG_TESTING, "text = %s\n", text);
 					im->produceSementicErrorString(_("double \"duration\""));
 					return;
 				     }
@@ -1192,11 +1173,11 @@ unsigned int NedMusicXMLImport::determineState() {
 	char Str[DEFAULT_STRING_LENGTH];
 	unsigned int state = 0;
 	switch (m_alter) {
-		case -2: state |= STAT_DFLAT; printf("STAT_DFLAT\n"); fflush(stdout); break;
+		case -2: state |= STAT_DFLAT; NedResource::DbgMsg(DBG_TESTING, "STAT_DFLAT\n"); break;
 		case -1: state |= STAT_FLAT; break;
 		case  0: state |=  STAT_NATURAL; break;
 		case  1: state |=  STAT_SHARP; break;
-		case  2: state |=  STAT_DSHARP; printf("STAT_DSHARP\n"); fflush(stdout);break;
+		case  2: state |=  STAT_DSHARP; NedResource::DbgMsg(DBG_TESTING, "STAT_DSHARP\n"); break;
 		case UNKNOWN_LINE: break;
 		default:
 			sprintf(Str, _("unsupported alter value %d"), m_alter);
@@ -1206,6 +1187,29 @@ unsigned int NedMusicXMLImport::determineState() {
 	if (m_staccato) {
 		state |= STAT_STACC;
 	}
+	if (m_staccatissimo) {
+		state |= STAT_STACCATISSIMO;
+	}
+	if (m_strong_accent) {
+		state |= STAT_SFORZATO;
+	}
+	if (m_tenuto) {
+		state |= STAT_TENUTO;
+	}
+	if (m_accent) {
+		state |= STAT_SFORZANDO;
+	}
+	if (m_trill) {
+		state |= STAT_TRILL;
+	}
+	if (m_pedal_off) {
+		m_pedal_off = false;
+		state |= STAT_PED_OFF;
+	}
+	if (m_pedal_on) {
+		m_pedal_on = false;
+		state |= STAT_PED_ON;
+	}
 	return state;
 }
 
@@ -1243,7 +1247,7 @@ void NedMusicXMLImport::determineUpBeat() {
 	if (voice_duration < measure_length) {
 		if (m_upbeat_inverse != 1) {
 			if (m_upbeat_inverse != measure_length - voice_duration) {
-				printf("contr m_upbeat_inverse\n"); fflush(stdout);
+				NedResource::DbgMsg(DBG_TESTING, "contr m_upbeat_inverse\n"); 
 			}
 		}
 		else {
@@ -1273,13 +1277,6 @@ void NedMusicXMLImport::removeEmptyParts() {
 	while (deleted);
 }
 
-void NedMusicXMLImport::resetTaken() {
-	int i;
-
-	for (i = 0; i < m_partcount; i++) {
-		m_parts[i].resetTaken();
-	}
-}
 				
 void NedMusicXMLImport::tie(NedChordOrRest *chord, NedNote *note) {
 	GList *lptr1, *lptr2;
