@@ -48,6 +48,12 @@
 #define TENOR_PITCH_OFFS  27
 #define SOPRAN_PITCH_OFFS 33
 
+#define TREBLE_LINE_OFFS  0
+#define BASS_LINE_OFFS   12
+#define ALTO_LINE_OFFS    6
+#define TENOR_LINE_OFFS   8
+#define SOPRAN_LINE_OFFS  2
+
 struct pitch_descr {
 	char pitch, min_sharp, min_flat;
 };
@@ -64,10 +70,26 @@ pitch_descr NedNote::lineToPitch[] = {
 { 96 /* C */, 2, 6}, { 98 /* D */, 4, 4}, {100 /* E */, 6, 2}, {101 /* F */, 1, 100}, {103 /* G */, 3, 5}, {105 /* A */, 5, 3}, {107 /* B */, 100, 1},
 {108 /* C */, 2, 6}, {110 /* D */, 4, 4}, {112 /* E */, 6, 2}, {114 /* F */, 1, 100}, {115 /* G */, 3, 5}, {117 /* A */, 5, 3}, {119 /* B */, 100, 1}};
 
+int NedNote::signToBase[][2] = {
+	{-6, 66}, {-5, 61}, {-4, 68}, {-3, 63}, {-2, 70 }, {-1, 65}, {0, 60}, {1, 67}, {2, 62}, {3, 69}, {4, 64}, {5, 71}, {6, 66}
+};
+
+int NedNote::diaDists[] = {-1, 0, 2, 4, 5, 7, 9, 11, 12, 14};
+
 NedNote::NedNote(NedChordOrRest *chord_or_rest, int line, int head, unsigned int status) :
 m_active(FALSE), m_acc_dist(ACCIDENTAL_BASE_DIST), m_line(line), m_tmp_line(UNKNOWN_LINE), m_status(status), m_head(head), m_chord_or_rest(chord_or_rest),
 m_tie_forward(NULL), m_tie_backward(NULL){
 	m_ypos = - line * LINE_DIST/2.0;
+	/*
+	NedResource::DbgMsg(DBG_TESTING, "keysig = 0, pitch = 62, pitchup = %d, pitchdown = %d\n", findPitchUpDown(0, 62, true),  findPitchUpDown(0, 62, false));
+	NedResource::DbgMsg(DBG_TESTING, "keysig = 0, pitch = 64, pitchup = %d, pitchdown = %d\n", findPitchUpDown(0, 64, true),  findPitchUpDown(0, 64, false));
+	NedResource::DbgMsg(DBG_TESTING, "keysig = 0, pitch = 65, pitchup = %d, pitchdown = %d\n", findPitchUpDown(0, 65, true),  findPitchUpDown(0, 65, false));
+	NedResource::DbgMsg(DBG_TESTING, "keysig = 2, pitch = 64, pitchup = %d, pitchdown = %d\n", findPitchUpDown(2, 64, true),  findPitchUpDown(2, 64, false));
+	NedResource::DbgMsg(DBG_TESTING, "keysig = 2, pitch = 66, pitchup = %d, pitchdown = %d\n", findPitchUpDown(2, 66, true),  findPitchUpDown(2, 66, false));
+	NedResource::DbgMsg(DBG_TESTING, "keysig = 2, pitch = 67, pitchup = %d, pitchdown = %d\n", findPitchUpDown(2, 67, true),  findPitchUpDown(2, 67, false));
+	NedResource::DbgMsg(DBG_TESTING, "keysig = -4, pitch = 68, pitchup = %d, pitchdown = %d\n", findPitchUpDown(-4, 68, true),  findPitchUpDown(-4, 68, false));
+	NedResource::DbgMsg(DBG_TESTING, "keysig = -4, pitch = 67, pitchup = %d, pitchdown = %d\n", findPitchUpDown(-4, 67, true),  findPitchUpDown(-4, 67, false));
+	*/
 }
 
 NedMainWindow *NedNote::getMainWindow() {return m_chord_or_rest->getStaff()->getSystem()->getPage()->getMainWindow();}
@@ -209,13 +231,25 @@ void NedNote::draw(cairo_t *cr) {
 	}
 	if (m_tie_forward != NULL) {
 		if (m_chord_or_rest->getVoice() == m_tie_forward->getVoice()) {
-			if (!m_chord_or_rest->hasUpDir() && !m_tie_forward->getChord()->hasUpDir()) {
-				tie_y_offs1 = -TIE_Y_OFFS1;
-				tie_y_offs2 = -TIE_Y_OFFS2;
+			if (getVoice()->getStemDir() == STEM_DIR_NONE) {
+				if (!m_chord_or_rest->hasUpDir() && !m_tie_forward->getChord()->hasUpDir()) {
+					tie_y_offs1 = -TIE_Y_OFFS1;
+					tie_y_offs2 = -TIE_Y_OFFS2;
+				}
+				else  {
+					tie_y_offs1 = TIE_Y_OFFS1;
+					tie_y_offs2 = TIE_Y_OFFS2;
+				}
 			}
-			else  {
-				tie_y_offs1 = TIE_Y_OFFS1;
-				tie_y_offs2 = TIE_Y_OFFS2;
+			else {
+				if (getVoice()->getStemDir() == STEM_DIR_UP) {
+					tie_y_offs1 = -TIE_Y_OFFS1;
+					tie_y_offs2 = -TIE_Y_OFFS2;
+				}
+				else  {
+					tie_y_offs1 = TIE_Y_OFFS1;
+					tie_y_offs2 = TIE_Y_OFFS2;
+				}
 			}
 			px0 = m_chord_or_rest->getXPos() + m_xpos + TIE_X_START;
 			px1 = m_tie_forward->m_chord_or_rest->getXPos() + m_tie_forward->m_xpos - TIE_X_START;
@@ -259,6 +293,35 @@ void NedNote::draw(cairo_t *cr) {
 	}
 };
 
+bool NedNote::isFirstNote() {
+	return m_chord_or_rest->getFirstNote() == this;
+}
+
+
+int NedNote::findPitchUpDown(int key, int pitch, bool up) {
+	int base = 60;
+	int i;
+
+	for (i = 0; i < 12; i++) {
+		if (signToBase[i][0] == key) {
+			base = signToBase[i][1];
+			break;
+		}
+	}
+	while (pitch > base + 12) base += 12;
+	while (pitch < base) base -= 12;
+	for (i = 1; i < 9; i++) {
+		if (base + diaDists[i] == pitch) {
+			if (up) {
+				return base + diaDists[i+1];
+			}
+			return base + diaDists[i-1];
+		}
+	}
+	if (up) return pitch + 1;
+	return pitch - 1;
+}
+
 bool NedNote::testForTiesToDelete(NedCommandList *command_list, unsigned int dir /* = BREAK_TIE_BACKWARD */, bool execute /* = false */) {
 	bool deleted_ties = FALSE;
 	NedUnTieForwardCommand *untie_cmd;
@@ -311,15 +374,73 @@ void NedNote::setOffset(char offs_array[115]) {
 				break;
 	}
 }
-	
 
-int NedNote::getPitch(int clef, int keysig, int octave_shift) {
+int NedNote::pitchToLine(int pitch, int clef, int keysig, int octave_shift, int *offs) {
+	int oct = 0;
+	int line;
+	*offs = 0;
+	int clef_offs = 0;
+
+	while (pitch < 60) {
+		pitch += 12; oct--;
+	}
+	while (pitch >= 72) {
+		pitch-= 12; oct++;
+	}
+	switch (clef) {
+		case NEUTRAL_CLEF1:
+		case TREBLE_CLEF: clef_offs = TREBLE_LINE_OFFS; break;
+		case NEUTRAL_CLEF2:
+		case BASS_CLEF: clef_offs = BASS_LINE_OFFS; break;
+		case ALTO_CLEF: clef_offs = ALTO_LINE_OFFS; break;
+		case TENOR_CLEF: clef_offs = TENOR_LINE_OFFS; break;
+		case SOPRAN_CLEF: clef_offs = SOPRAN_LINE_OFFS; break;
+	}
+
+	if (keysig < 0) {
+		switch (pitch) {
+			case 60: line = -2; break;
+			case 61: line = -1; *offs = -1; break;
+			case 62: line = -1; break;
+			case 63: line =  0; *offs = -1; break;
+			case 64: line =  0; break;
+			case 65: line =  1; break;
+			case 66: line =  2; *offs = -1; break;
+			case 67: line =  2; break;
+			case 68: line =  3; *offs = -1; break;
+			case 69: line =  3; break;
+			case 70: line =  4; *offs = -1; break;
+			case 71: line =  4; break;
+		}
+	}
+	else {
+		switch (pitch) {
+			case 60: line = -2; break;
+			case 61: line = -2; *offs = 1; break;
+			case 62: line = -1; break;
+			case 63: line = -1; *offs = 1; break;
+			case 64: line =  0; break;
+			case 65: line =  1; break;
+			case 66: line =  1; *offs = 1; break;
+			case 67: line =  2; break;
+			case 68: line =  2; *offs = 1; break;
+			case 69: line =  3; break;
+			case 70: line =  3; *offs = 1; break;
+			case 71: line =  4; break;
+		}
+	}
+
+	return line + 7 * (oct - octave_shift) + clef_offs;
+}
+
+int NedNote::getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bool *flat) {
 	int p;
 	int offs;
 	char offs_array[115];
+	*flat = false;
 
 	if (m_tie_backward != 0) {
-		return m_tie_backward->getPitch(clef, keysig, octave_shift);
+		return m_tie_backward->getPitch(clef, keysig, octave_shift, pitch_offs, flat);
 	}
 
 	switch (clef) {
@@ -335,9 +456,9 @@ int NedNote::getPitch(int clef, int keysig, int octave_shift) {
 	memset(offs_array, 0, sizeof(offs_array));
 	getStaff()->findAccidentals(offs_array, getSystem()->getMeasure(m_chord_or_rest->getMidiTime()), m_chord_or_rest->getMidiTime());
 	switch (m_status & ACCIDENTAL_MASK) {
-		case STAT_DFLAT: p -= 2;
+		case STAT_DFLAT: p -= 2; *flat = true;
 				break;
-		case STAT_FLAT: p--;
+		case STAT_FLAT: p--; *flat = true;
 				break;
 		case STAT_SHARP: p++;
 				break;
@@ -351,6 +472,7 @@ int NedNote::getPitch(int clef, int keysig, int octave_shift) {
 			if (keysig < 0) {
 				if (lineToPitch[m_line+offs].min_flat <= -keysig && offs_array[m_line+offs] == 0) {
 					p--;
+					*flat = true;
 					break;
 				}
 			}
@@ -359,11 +481,13 @@ int NedNote::getPitch(int clef, int keysig, int octave_shift) {
 				break;
 			}
 			p += offs_array[m_line+offs];
+			if (offs_array[m_line+offs] < 0) *flat = true;
 			break;
 	} 
-	return p + octave_shift;
+	return p + octave_shift + pitch_offs;
 }
 
+	
 void NedNote::removeUnneededAccidental(int clef, int keysig, int octave_shift) {
 	int p;
 	int offs;
@@ -386,9 +510,11 @@ void NedNote::removeUnneededAccidental(int clef, int keysig, int octave_shift) {
 					 m_status &= (~(ACCIDENTAL_MASK));
 					 break;
 				}
-				if (keysig < 0) {
-					if (lineToPitch[m_line+offs].min_flat <= -keysig && offs_array[m_line+offs] != 2 /* natural */) {
-						m_status &= (~(ACCIDENTAL_MASK));
+				if (offs_array[m_line+offs] != VIRTUAL_NATURAL_OFFS) {
+					if (keysig < 0) {
+						if (lineToPitch[m_line+offs].min_flat <= -keysig && offs_array[m_line+offs] != 2 /* natural */) {
+							m_status &= (~(ACCIDENTAL_MASK));
+						}
 					}
 				}
 				break;
@@ -396,8 +522,10 @@ void NedNote::removeUnneededAccidental(int clef, int keysig, int octave_shift) {
 					m_status &= (~(ACCIDENTAL_MASK));
 					break;
 				 }
-				if (lineToPitch[m_line+offs].min_sharp <= keysig && offs_array[m_line+offs] != 2 /* natural */) {
-					m_status &= (~(ACCIDENTAL_MASK));
+				if (offs_array[m_line+offs] != VIRTUAL_NATURAL_OFFS) {
+					if (lineToPitch[m_line+offs].min_sharp <= keysig && offs_array[m_line+offs] != 2 /* natural */) {
+						m_status &= (~(ACCIDENTAL_MASK));
+					}
 				}
 				break;
 		case STAT_NATURAL: 
@@ -418,27 +546,131 @@ void NedNote::removeUnneededAccidental(int clef, int keysig, int octave_shift) {
 				break;
 	} 
 }
-	
 
-void NedNote::prepareReplay(int clef, int keysig, int octave_shift, int grace_time, bool is_grace, bool is_stacc) {
-	int pitch;
 
-	pitch = getPitch(clef, keysig, octave_shift);
+unsigned int NedNote::determineState(int pitch, int line, int half_offs, char offs_array[117], int keysig) {
+	int i;
+
+	switch (half_offs) {
+		case -1: if (offs_array[line + 115 / 2] == -1) return 0;
+			 if (offs_array[line + 115 / 2] != VIRTUAL_NATURAL_OFFS) {
+			 	if (keysig < 0) {
+			 		for (i = sizeof(lineToPitch) / sizeof(pitch_descr) - 1; i >= 0; i--) {
+						if (lineToPitch[i].pitch < pitch) {
+							i++; break;
+						}
+					}
+			 		if (lineToPitch[i].min_flat <= -keysig) return 0;
+			 	}
+			 }
+			 offs_array[line + 115 / 2] = -1;
+			 return STAT_FLAT;
+		case  1: if (offs_array[line + 115 / 2] == 1) return 0;
+			 if (offs_array[line + 115 / 2] != VIRTUAL_NATURAL_OFFS) {
+			 	if (keysig > 0) {
+				  for ( i = 0; i < (signed int)(sizeof(lineToPitch) / sizeof(pitch_descr)); i++) {
+						if (lineToPitch[i].pitch > pitch) {
+							i--; break;
+						}
+					}
+			 		if (lineToPitch[i].min_sharp <= keysig) return 0;
+			 	}
+			 }
+			 offs_array[line + 115 / 2] =  1;
+			 return STAT_SHARP;
+		default: if (offs_array[line + 115 / 2] == VIRTUAL_NATURAL_OFFS) return 0;
+			 if (offs_array[line + 115 / 2] == 1 || offs_array[line + 115 / 2] == -1) {
+			 	offs_array[line + 115 / 2] = VIRTUAL_NATURAL_OFFS;
+				return STAT_NATURAL;
+			 }
+			 if (keysig < 0) {
+			 	for (i = sizeof(lineToPitch) / sizeof(pitch_descr) - 1; i >= 0; i--) {
+					if (lineToPitch[i].pitch < pitch) {
+						i++; break;
+					}
+				}
+			 	if (lineToPitch[i].min_flat <= -keysig) {
+					offs_array[line + 115 / 2] = VIRTUAL_NATURAL_OFFS;
+					return STAT_NATURAL;
+				}
+			 }
+			 if (keysig > 0) {
+			 	for ( i = 0; i < (signed int)(sizeof(lineToPitch) / sizeof(pitch_descr)); i++) {
+					if (lineToPitch[i].pitch > pitch) {
+						i--; break;
+					}
+				}
+			 	if (lineToPitch[i].min_sharp <= keysig) {
+					offs_array[line + 115 / 2] = VIRTUAL_NATURAL_OFFS; // neutral
+					return STAT_NATURAL;
+				}
+			}
+			break;
+	} 
+	return 0;
+}
+
+void NedNote::prepareReplay(unsigned long long midi_start_time, int clef, int keysig, int octave_shift, int pitch_offs, 
+		 unsigned int grace_time, bool is_grace, bool is_stacc) {
+	int pitch, pitch2, tt;
+	unsigned int i;
+	bool dummy;
+
+	pitch = getPitch(clef, keysig, octave_shift, pitch_offs, &dummy);
 	if (is_grace) {
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			m_chord_or_rest->getMidiTime() - grace_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_volume, this);
+			midi_start_time - grace_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
 			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
 	}
 	else if (is_stacc && m_chord_or_rest->getLength() > NOTE_32 && grace_time == 0) {
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			m_chord_or_rest->getMidiTime(), SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_volume, this);
+			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
 			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() / 2, SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
 	}
+	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_TRILL) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
+		pitch2 = findPitchUpDown(keysig, pitch, true);
+		for (tt = 0, i = 0; i < m_chord_or_rest->getDuration() / NOTE_32; i++, tt = 1 - tt) {
+			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+			midi_start_time + i * NOTE_32, SND_SEQ_EVENT_NOTEON, tt ? pitch : pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+			m_chord_or_rest->getMidiTime() + (i + 1) * NOTE_32, SND_SEQ_EVENT_NOTEOFF, tt ? pitch : pitch2, 0, this);
+		}
+	}
+	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_PRALL) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
+		pitch2 = findPitchUpDown(keysig, pitch, true);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+	}
+	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_MORDENT) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
+		pitch2 = findPitchUpDown(keysig, pitch, false);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+	}
 	else {
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			m_chord_or_rest->getMidiTime(), SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_volume, this);
+			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
 			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() - grace_time, SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
 	}
@@ -448,6 +680,19 @@ void NedNote::setTieSimple(NedNote *other_note) {
 	m_tie_forward = other_note;
 	other_note->m_tie_backward = this;
 }
+
+void NedNote::setTieIfSamePitch(NedNote *other_note) {
+	if (m_line != other_note->m_line) return;
+	if ((m_status & ACCIDENTAL_MASK) != (other_note->m_status & ACCIDENTAL_MASK)) return;
+	m_tie_forward = other_note;
+	other_note->m_tie_backward = this;
+}
+
+bool NedNote::isTheSame(NedNote *other_note) {
+	if (m_line != other_note->m_line) return false;
+	if ((m_status & ACCIDENTAL_MASK) != (other_note->m_status & ACCIDENTAL_MASK)) return false;
+	return true;
+}
 	
 void NedNote::removeBackwardTie() {
 	if (m_tie_backward == NULL) return;
@@ -455,6 +700,12 @@ void NedNote::removeBackwardTie() {
 	m_tie_backward = NULL;
 }
 	
+void NedNote::removeForwardTie() {
+	if (m_tie_forward == NULL) return;
+	m_tie_forward->m_tie_backward = NULL;
+	m_tie_forward = NULL;
+}
+	
 
 void NedNote::do_tie(NedNote *other_note) {
 	int sorting;
@@ -527,16 +778,18 @@ bool NedNote::testYShift(double y) {
 
 void NedNote::setStatus(unsigned int status) {
 	int clef, keysig, octave_shift;
+	bool dummy;
 	staff_context_str *context;
 	m_status = status;
 	context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
 	getStaff()->getCurrentClefAndKeysig(m_chord_or_rest->getMidiTime(), &clef, &keysig, &octave_shift);
 	if (NedResource::m_midi_echo) {
-		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, getPitch(clef, keysig, octave_shift), context->m_midi_volume);
+		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, getPitch(clef, keysig, octave_shift, 0, &dummy), context->m_midi_volume);
 	}
 }
 void NedNote::moveUpDown(int line) {
 	staff_context_str *context;
+	bool dummy;
 	int clef, keysig, octave_shift;
 	getStaff()->getCurrentClefAndKeysig(m_chord_or_rest->getMidiTime(), &clef, &keysig, &octave_shift);
 	m_line = line;
@@ -545,7 +798,7 @@ void NedNote::moveUpDown(int line) {
 	getMainWindow()->reposit(NULL, getPage());
 	context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
 	if (NedResource::m_midi_echo) {
-		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, getPitch(clef, keysig, octave_shift), context->m_midi_volume);
+		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, getPitch(clef, keysig, octave_shift, 0, &dummy), context->m_midi_volume);
 	}
 	m_tmp_line = UNKNOWN_LINE;
 }
@@ -704,5 +957,45 @@ void NedNote::adjust_pointers(struct addr_ref_str *addrlist) {
 	}
 }
 		
-	
+void NedNote::exportLilyPond(FILE *fp, int *last_line, bool *tie) {
+	int clef, keysig, octave_shift;
+	int pitch;
+	int line;
+	bool flat;
+	staff_context_str *context;
+	context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
+	getStaff()->getCurrentClefAndKeysig(m_chord_or_rest->getMidiTime(), &clef, &keysig, &octave_shift);
+	if (clef == NEUTRAL_CLEF1 || clef == NEUTRAL_CLEF2) clef = ALTO_CLEF;
+	pitch = getPitch(clef, keysig, octave_shift, 0, &flat);
 
+	while (pitch < 60) pitch += 12;
+	while (pitch > 71) pitch -= 12;
+
+	switch (pitch) {
+		case 60: fprintf(fp, " c"); break;
+		case 61: fprintf(fp, " %s", flat ? "des" : "cis"); break;
+		case 62: fprintf(fp, " d"); break;
+		case 63: fprintf(fp, " %s", flat ? "es" : "dis"); break;
+		case 64: fprintf(fp, " e"); break;
+		case 65: fprintf(fp, " f"); break;
+		case 66: fprintf(fp, " %s", flat ? "ges" : "fis"); break;
+		case 67: fprintf(fp, " g"); break;
+		case 68: fprintf(fp, " %s", flat ? "as" : "gis"); break;
+		case 69: fprintf(fp, " a"); break;
+		case 70: fprintf(fp, " %s", flat ? "bes" : "ais"); break;
+		case 71: fprintf(fp, " b"); break;
+	}
+	line = m_line;
+	while (line > *last_line + 3) {
+		fprintf(fp, "'");
+		line -= 7;
+	}
+	while (line < *last_line - 3) {
+		fprintf(fp, ",");
+		line += 7;
+	}
+	if (m_tie_forward != NULL) {
+		*tie = true;
+	}
+	*last_line = m_line;
+}
