@@ -20,6 +20,7 @@
 /****************************************************************************************/
 
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 #include "page.h"
 #include "system.h"
 #include "mainwindow.h"
@@ -29,13 +30,32 @@
 #include "appendsystemcommand.h"
 #include "movesystemcommand.h"
 #include "removepagecommand.h"
+#include "insertfreeplaceablecommand.h"
 #include "getsystemfromnextpagecmmand.h"
+#include "commandhistory.h"
 #include "commandlist.h"
+#include "chordorrest.h"
+#include "volumesign.h"
+#include "temposign.h"
 #include "scoreinfodialog.h"
+#include "slurpoint.h"
+#include "slur.h"
+#include "linepoint.h"
+#include "crescendo.h"
+#include "linesdialog.h"
+#include "octavation.h"
+#include "acceleration.h"
+#include "freesign.h"
 
 
 #define X_POS_INVERS(p) ((leftx + (p)) / zoom_factor - m_xpos)
 #define Y_POS_INVERS(p) ((topy + (p)) / zoom_factor)
+#define X_POS_INVERS_PAGE_REL(p) ((leftx + (p)) / zoom_factor - getContetXpos())
+
+#define X_POS(p) (((p) + m_xpos) * zoom_factor - leftx)
+#define Y_POS(p) ((p) * zoom_factor - topy)
+
+#define X_POS_PAGE_REL(p) (((p) + getContetXpos()) * zoom_factor - leftx)
 
 #define DEFAULT_SYSTEM_DIST (2 * 5 * LINE_DIST)
 NedPage::NedPage(NedMainWindow *main_window, double width, double height, int nr, unsigned int start_measure_number, bool start) :
@@ -89,6 +109,7 @@ NedSystem *NedPage::appendSystem() {
 	                                m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER), system_nr, 1, TRUE));
 
 	placeStaffs(0);
+
 	return system;
 }
 
@@ -110,7 +131,6 @@ void NedPage::insertSystem(NedSystem *system) {
 	
 	m_systems = g_list_prepend(m_systems, system);
 	system->changePageInfo(this);
-
 }
 
 bool NedPage::isPageOverflow() {
@@ -186,6 +206,22 @@ bool NedPage::isLast(NedSystem *system) {
 }
 
 
+bool NedPage::find_staff_and_line(int x, int y, NedStaff **last_staff, int *last_line) {
+	double zoom_factor = m_main_window->getCurrentZoomFactor();
+	double leftx = m_main_window->getLeftX();
+	double bottom;
+
+	double xl = X_POS_INVERS(x);
+	double yl2;
+
+
+	if (xl < 0.0 || xl > m_width) return false;
+
+
+	return findLine(x, y, &yl2, last_line, &bottom, last_staff);
+}
+
+
 
 void NedPage::draw(cairo_t *cr, double main_width, double main_height) {
 	double leftx = m_main_window->getLeftX();
@@ -317,13 +353,87 @@ bool NedPage::reposit(int pass, NedCommandList *command_list /* = NULL */) {
 	return changes;
 }
 
-void NedPage::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array) {
+void NedPage::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, bool first) {
+	GList *lptr;
+	bool first_system = true;
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->setAndUpdateClefTypeAndKeySig(clef_and_key_array, first && first_system);
+		first_system = false;
+	}
+}
+
+void NedPage::determineVolume(unsigned int *vol, NedChordOrRest *element, int staff_nr) {
+	GList *lptr;
+	for (lptr = g_list_first(m_systems); lptr && *vol == 256; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->determineVolume(vol, element, staff_nr);
+	}
+}
+
+void NedPage::determineTempoInverse(NedChordOrRest *element, NedSystem* system, double *tempoinverse, bool *found) {
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		((NedSystem *) lptr->data)->setAndUpdateClefTypeAndKeySig(clef_and_key_array);
+		if ((NedSystem *) lptr->data == system) {
+			((NedSystem *) lptr->data)->determineTempoInverse(element, element->getMidiTime(), tempoinverse, found);
+			break;
+		}
+		((NedSystem *) lptr->data)->determineTempoInverse(NULL, element->getMidiTime(), tempoinverse, found);
 	}
 }
 
+void NedPage::detectVoices(int staff_nr, unsigned int *voice_mask, NedSystem **last_system,  unsigned long long *end_time) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->detectVoices(staff_nr, voice_mask, last_system, end_time);
+	}
+}
+
+void NedPage::cutEmptyVoices() {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->cutEmptyVoices();
+	}
+}
+
+bool NedPage::hasOnlyRests() {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		if (!((NedSystem *) lptr->data)->hasOnlyRests()) return false;
+	}
+	return true;
+}
+
+void NedPage::testTies() {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->testTies();
+	}
+}
+
+bool NedPage::exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
+	NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->exportLilyPond(fp, staff_nr, voice_nr, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map);
+		if ((NedSystem *) lptr->data == last_system) return false;
+	}
+	return true;
+}
+
+bool NedPage::exportLilyLyrics(FILE *fp, int staff_nr, int voice_nr, int line_nr, NedSystem *last_system, unsigned long long end_time, int *sil_count) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->exportLilyLyrics(fp, ((NedSystem *) lptr->data) == last_system, staff_nr, voice_nr, line_nr, end_time, sil_count);
+		if (((NedSystem *) lptr->data) == last_system) return false;
+	}
+	return true;
+}
+
 void NedPage::handleStaffElements() {
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
@@ -361,6 +471,54 @@ void NedPage::reconfig_paper(double width, double height) {
 
 }
 
+void NedPage::getLinePoints(NedStaff *start_staff, NedStaff *end_staff, GList **staves) {
+	GList *lptr;
+	NedStaff *staff;
+	NedPage *next_page;
+	int staff_nr = start_staff->getStaffNumber();
+	int i;
+
+
+	if (start_staff->getPage() != this) {
+		if ((lptr = g_list_find(next_page->m_systems, start_staff->getSystem())) == NULL) {
+			NedResource::Abort("NedPage::getLinePoints: error 1");
+		}
+	}
+
+	if ((lptr = g_list_find(m_systems, start_staff->getSystem())) == NULL) {
+		NedResource::Abort("NedPage::getLinePoints: error 2");
+	}
+
+
+	for (i = 0; i < 2; i++) {
+		for (; lptr; lptr = g_list_next(lptr)) {
+			if ((staff = ((NedSystem *) lptr->data)->getStaff(staff_nr)) == NULL) {
+				NedResource::Abort("NedPage::getLinePoints: error 3");
+			}
+			*staves = g_list_append(*staves, staff);
+			if (staff == end_staff) return;
+		}
+	
+	
+		if ((next_page = end_staff->getPage()) == NULL) {
+			NedResource::Abort("NedPage::getLinePoints: error 4");
+		}
+		if ((lptr = g_list_first(next_page->m_systems)) == NULL) {
+			NedResource::Abort("NedPage::getLinePoints: error 5");
+		}
+	}
+
+	NedResource::DbgMsg(DBG_CRITICAL, "gesucht wurde in staff %d\n", staff_nr);
+
+	NedResource::Abort("NedPage::getLinePoints: error 6");
+}
+
+
+		
+
+
+
+
 void NedPage::savePage(FILE *fp) {
 	GList *lptr;
 	fprintf(fp, "PAGE %d\n", m_page_number);
@@ -387,7 +545,7 @@ void NedPage::restorePage(FILE *fp, unsigned int start_measure_number) {
 				NedResource::unreadWord(buffer);
 				return;
 			}
-			printf("buffer = %s\n", buffer);
+			NedResource::DbgMsg(DBG_CRITICAL, "buffer = %s\n", buffer);
 			NedResource::m_error_message = "(2)SYSTEM or PAGE expected";
 			return;
 		}
@@ -430,7 +588,7 @@ bool NedPage::tryChangeLength(NedChordOrRest *chord_or_rest) {
 }
 
 
-bool NedPage::trySelect(double x, double y) {
+bool NedPage::trySelect(double x, double y, bool only_free_placeables /* = false */) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
 	double leftx = m_main_window->getLeftX();
 
@@ -441,13 +599,26 @@ bool NedPage::trySelect(double x, double y) {
 
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		if (((NedSystem *) lptr->data)->trySelect(x, y)) {
+		if (((NedSystem *) lptr->data)->trySelect(x, y, only_free_placeables)) {
 			return TRUE;
 		}
 	}
 	return FALSE;
 }
 
+NedChordOrRest *NedPage::findNearestElement(int staff_nr, double x, double y, NedStaff *oldstaff, double *ydist) {
+	double mindist = 10000000.0;
+	NedChordOrRest *nearestElement = NULL, *element;
+
+	GList *lptr;
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		if ((element = ((NedSystem *) lptr->data)->findNearestElement(staff_nr, x, y, oldstaff, &mindist, ydist)) != NULL) {
+			nearestElement = element;
+		}
+	}
+	return nearestElement;
+}
+
 bool NedPage::findTimeOfMeasure(int meas_num, unsigned long long *meas_time, unsigned long long *system_offs) {
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
@@ -561,17 +732,154 @@ NedStaff *NedPage::findStaff(double x, double y, NedMeasure **measure) {
 	return NULL;
 }
 
-bool NedPage::tryInsertOrErease(double x, double y) {
+double NedPage::convertX(double xpos) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
 	double leftx = m_main_window->getLeftX();
 
+	return X_POS_PAGE_REL(xpos);
+}
+	
+
+bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*  = NULL */, bool force_rest /* = false */) {
+	double zoom_factor = m_main_window->getCurrentZoomFactor();
+	double leftx = m_main_window->getLeftX();
+	NedChordOrRest *element, *element1, *element2;
+	double dummy, dummy0;
+	double topy, xx, yy, yyy;
+	unsigned int kind, tempo;
+	NedSlurPoint *sp[3];
+	NedCommandList *command_list;
+	NedLinePoint *lp0, *lp1, *lp2;
+	if (newObj != NULL) {
+		*newObj = NULL;
+	}
+
 	double xl = X_POS_INVERS(x);
 
 
+#define LINE_START_X_DIST 2.0
+
 	if (xl < 0.0 || xl > m_width) return FALSE;
 
 	GList *lptr;
 	double mindist, d;
+
+	switch(m_main_window->getSpecialType()) {
+	case TYPE_DYNAMIC:
+	case TYPE_TEMPO:
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		yy = Y_POS_INVERS(y);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		command_list = new NedCommandList(m_main_window);
+		if (m_main_window->getSpecialType() == TYPE_DYNAMIC) {
+			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedVolumeSign(m_main_window->getSpecialSubType()), element, xx, yy));
+		}
+		else {
+			kind = (m_main_window->getSpecialSubType() & 0xffff) * FACTOR;
+			tempo = ((m_main_window->getSpecialSubType() >> 16) & 0xffff);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedTempoSign(kind, tempo), element, xx, yy));
+		}
+		command_list->execute();
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		return TRUE;
+	case TYPE_SIGN:
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		yy = Y_POS_INVERS(y);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		command_list = new NedCommandList(m_main_window);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeSign(m_main_window->getSpecialSubType()), element, xx, yy));
+		command_list->execute();
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		return TRUE;
+	case TYPE_FREE_TEXT:
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		yy = Y_POS_INVERS(y);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		command_list = new NedCommandList(m_main_window);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(m_main_window->m_freetext, element, xx, yy));
+		command_list->execute();
+		m_main_window->m_freetext = NULL;
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		return TRUE;
+	case TYPE_SLUR:
+#define SLUR_START_X_DIST 1.0
+#define SLUR_START_Y_DIST ( 2 * LINE_DIST )
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		if (xx > m_width / 2.0) xx -= 3 * SLUR_START_X_DIST;
+		yy = Y_POS_INVERS(y);
+		command_list = new NedCommandList(m_main_window);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		yyy = element->getSystem()->getYPos() + element->getStaff()->getMidPos();
+		command_list->addCommand(new NedInsertFreePlaceableCommand(sp[0] = new NedSlurPoint(NULL, 0), element, xx, yyy));
+		dummy0 = 10000000.0;
+		element1 = element->getStaff()->findNearestElement(xx + SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST, NULL, &dummy0, &dummy);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(sp[1] = new NedSlurPoint(NULL, 1), element1, xx + SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST));
+		dummy0 = 10000000.0;
+		element2 = element->getStaff()->findNearestElement(xx + 2 * SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST / 2, NULL, &dummy0, &dummy);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(sp[2] = new NedSlurPoint(NULL, 2), element2, xx + 2 * SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST / 2));
+		command_list->execute();
+		new NedSlur(sp);
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		m_main_window->m_selected_free_replaceable = sp[2];
+		return TRUE;
+	case TYPE_LINE:
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		if (xx > m_width / 2.0) xx -= 3 * SLUR_START_X_DIST;
+		yy = Y_POS_INVERS(y);
+		command_list = new NedCommandList(m_main_window);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(lp0 = new NedLinePoint(NULL, 0), element, xx, yy));
+		dummy0 = 10000000.0;
+		element = element->getStaff()->findNearestElement(xx + LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(lp1 = new NedLinePoint(NULL, 1), element, xx + LINE_START_X_DIST, yy));
+		command_list->execute();
+		switch (m_main_window->getSpecialSubType()) {
+			case LINE_CRESCENDO:
+			case LINE_DECRESCENDO:
+					new NedCrescendo(lp0, lp1, m_main_window->getSpecialSubType() == LINE_DECRESCENDO);
+					break;
+			case LINE_OCTAVATION1:
+			case LINE_OCTAVATION_1:
+			case LINE_OCTAVATION2:
+			case LINE_OCTAVATION_2:
+					new NedOctavation(lp0, lp1, m_main_window->getSpecialSubType());
+					break;
+					
+		}
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		m_main_window->m_selected_free_replaceable = lp0;
+		return TRUE;
+	case TYPE_LINE3:
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		if (xx > m_width / 2.0) xx -= 3 * SLUR_START_X_DIST;
+		yy = Y_POS_INVERS(y);
+		command_list = new NedCommandList(m_main_window);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(lp0 = new NedLinePoint(NULL, 0), element, xx, yy));
+		dummy0 = 10000000.0;
+		element = element->getStaff()->findNearestElement(xx + LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(lp1 = new NedLinePoint(NULL, 1), element, xx + LINE_START_X_DIST, yy));
+		dummy0 = 10000000.0;
+		element = element->getStaff()->findNearestElement(xx + 2 * LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(lp2 = new NedLinePoint(NULL, 1), element, xx + 2 * LINE_START_X_DIST, yy));
+		command_list->execute();
+		new NedAcceleration(lp0, lp1, lp2, m_main_window->getSpecialSubType() == LINE_RITARDANDO);
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		m_main_window->m_selected_free_replaceable = lp0;
+		return TRUE;
+	}
 	lptr = g_list_first(m_systems);
 	NedSystem *nearest_system = (NedSystem *) lptr->data;
 	mindist = nearest_system->computeMidDist(y);
@@ -581,14 +889,14 @@ bool NedPage::tryInsertOrErease(double x, double y) {
 			mindist = d;
 		}
 	}
-	if (nearest_system->tryInsertOrErease(x, y)) {
+	if (nearest_system->tryInsertOrErease(x, y, newObj, force_rest)) {
 		return TRUE;
 	}
 	return FALSE;
 }
 
 
-bool NedPage::findLine(double x, double y, int *ypos, int *line, int *bottom) {
+bool NedPage::findLine(double x, double y, double *ypos, int *line, double *bottom, NedStaff **staff) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
 	double leftx = m_main_window->getLeftX();
 
@@ -606,7 +914,7 @@ bool NedPage::findLine(double x, double y, int *ypos, int *line, int *bottom) {
 			mindist = d;
 		}
 	}
-	if (nearest_system->findLine(x, y, ypos, line, bottom)) {
+	if (nearest_system->findLine(x, y, ypos, line, bottom, staff)) {
 		return TRUE;
 	}
 	return FALSE;
@@ -621,6 +929,14 @@ void NedPage::renumberSystems() {
 	}
 }
 
+NedPage *NedPage::getNextPage() {
+	return m_main_window->getNextPage(this);
+}
+
+NedPage *NedPage::getPreviousPage() {
+	return m_main_window->getPreviousPage(this);
+}
+
 void NedPage::findLastTouchedSystem(NedSystem **start_of_systems) {
 	GList *lptr;
 
@@ -636,7 +952,6 @@ bool NedPage::placeStaffs(int pass, NedCommandList *command_list /* = NULL */) {
 	GList *lptr;
 	NedPage *other_page;
 	double offs = STAFF_TOP_DIST;
-	NedRemovePageCommand *rem_last_page_cmd;
 	do_place_staffs(offs);
 	bool changed = FALSE;
 
