@@ -31,6 +31,9 @@
 #include "midiexport.h"
 #include "measure.h"
 #include "tuplet.h"
+
+#include "icons/nted.h"
+
 #include "icons/stroken_grace_icon.h"
 #include "icons/grace_eighth_icon.h"
 #include "icons/grace_sixth_icon.h"
@@ -67,6 +70,8 @@
 
 #include "icons/lyrics_icon.h"
 
+#include "icons/key_icon.h"
+
 #include "icons/normal_note_icon.h"
 #include "icons/drum1_icon.h"
 #include "icons/drum2_icon.h"
@@ -78,6 +83,7 @@
 #include "icons/dcross_icon.h"
 #include "icons/dflat_icon.h"
 #include "icons/stacc_icon.h"
+#include "icons/staccatissimo_icon.h"
 #include "icons/str_pizz_icon.h"
 #include "icons/tenuto_icon.h"
 #include "icons/sforzato_icon.h"
@@ -88,9 +94,23 @@
 #include "icons/brace_icon.h"
 #include "icons/bracket_icon.h"
 
+#include "icons/ped_on_icon.h"
+#include "icons/ped_off_icon.h"
+#include "icons/trill_icon.h"
+#include "icons/prall_icon.h"
+#include "icons/open_icon.h"
+#include "icons/mordent_icon.h"
+#include "icons/fermata_icon.h"
+#include "icons/arpeggio_icon.h"
+
+#include "icons/lines.h"
+#include "icons/signs.h"
+
 #define CONFIG_DIR ".nted"
 #define CONFIG_FILE "ntedrc"
 
+GdkPixbuf *NedResource::m_the_nted_icon;
+
 GdkPixbuf *NedResource::m_32ndnote_icon;
 GdkPixbuf *NedResource::m_64thnote_icon;
 GdkPixbuf *NedResource::m_eightnote_icon;
@@ -128,6 +148,8 @@ GdkPixbuf *NedResource::m_color_icon;
 
 GdkPixbuf *NedResource::m_lyrics_icon;
 
+GdkPixbuf *NedResource::m_key_icon;
+
 GdkPixbuf *NedResource::m_normal_note_icon;
 GdkPixbuf *NedResource::m_drum1_icon;
 GdkPixbuf *NedResource::m_drum2_icon;
@@ -139,6 +161,7 @@ GdkPixbuf *NedResource::m_drum6_icon;
 GdkPixbuf *NedResource::m_dcross_icon;
 GdkPixbuf *NedResource::m_dflat_icon;
 GdkPixbuf *NedResource::m_stacc_icon;
+GdkPixbuf *NedResource::m_staccatissimo_icon;
 GdkPixbuf *NedResource::m_str_pizz_icon;
 GdkPixbuf *NedResource::m_tenuto_icon;
 GdkPixbuf *NedResource::m_sforzato_icon;
@@ -149,6 +172,26 @@ GdkPixbuf *NedResource::m_bow_down_icon;
 GdkPixbuf *NedResource::m_brace_icon;
 GdkPixbuf *NedResource::m_bracket_icon;
 
+GdkPixbuf *NedResource::m_ped_on_icon;
+GdkPixbuf *NedResource::m_ped_off_icon;
+GdkPixbuf *NedResource::m_trill_icon;
+GdkPixbuf *NedResource::m_prall_icon;
+GdkPixbuf *NedResource::m_open_icon;
+GdkPixbuf *NedResource::m_mordent_icon;
+GdkPixbuf *NedResource::m_fermata_icon;
+GdkPixbuf *NedResource::m_arpeggio_icon;
+
+GdkPixbuf *NedResource::m_lines;
+GdkPixbuf *NedResource::m_signs;
+
+int NedResource::m_lines_buf_width;
+int NedResource::m_lines_buf_height;
+
+int NedResource::m_signs_buf_width;
+int NedResource::m_signs_buf_height;
+
+struct timeval NedResource::m_expected_time;
+
 snd_seq_t *NedResource::m_sequ = NULL;
 char NedResource::m_word_buffer[128];
 
@@ -183,12 +226,21 @@ int NedResource::m_last_immediate_channel = -1;
 bool NedResource::m_avoid_immadiate_play = FALSE;
 char *NedResource::m_recent_files[MAX_RECENT_FILES];
 const char *NedResource::m_print_cmd;
+int NedResource::m_width;
+int NedResource::m_height;
 GList *NedResource::m_temp_file_names = NULL;
 
 unsigned int NedResource::m_length_field_value = 0;
 
-char *NedResource::m_last_folder = NULL;
+double NedResource::m_tempo_inverse;
+unsigned long long NedResource::m_accel_midi_start;
+unsigned long long NedResource::m_accel_midi_end;
+double NedResource::m_accel_start_tempoinverse;
+double NedResource::m_accel_end_tempoinverse;
+
+const char *NedResource::m_last_folder = DATADIR "/nted/examples";
 char *NedResource::m_last_xml_dir = NULL;
+char *NedResource::m_last_midi_dir = NULL;
 
 const char *NedResource::GM_Instruments[] = {
  N_("Piano 1"),          N_("Piano 2"), N_("Piano 3"),          N_("Honky-tonk"),
@@ -274,6 +326,14 @@ cairo_font_options_t *NedResource::m_fontoptions;
 cairo_matrix_t NedResource::m_matrixes[ZOOMLEVELS];
 #endif
 
+struct fermata_info {
+	unsigned long long midi_time;
+	double tempo_inverse;
+	bool a_tempo;
+};
+
+GList *NedResource::m_fermata_list = NULL;
+
 
 void NedResource::initialize() {
 	int i;
@@ -282,6 +342,8 @@ void NedResource::initialize() {
 	GtkStockItem stock_item_down;
 	GtkStockItem stock_item_delete_staff;
 	m_midi_echo = TRUE;
+	m_height = HEIGHT;
+	m_width = WIDTH;
 	const char *fontdir;
 
 	for (i = 0; i < MAX_RECENT_FILES; m_recent_files[i++] = NULL);
@@ -297,6 +359,8 @@ void NedResource::initialize() {
 	sprintf(m_font_file_name, "%s/%s", fontdir, FONTFILE);
 	init_fonts(m_font_file_name);
 	m_icon_factory = gtk_icon_factory_new();
+	loadPixmap(nted, &m_the_nted_icon, "the-nted-icon", &dummyw, &dummyh);
+
 	loadPixmap(n32ndnote_icon, &m_32ndnote_icon, "32ndnote-icon", &dummyw, &dummyh);
 	loadPixmap(n64thnote_icon, &m_64thnote_icon, "64thnote-icon", &dummyw, &dummyh);
 	loadPixmap(eightnote_icon, &m_eightnote_icon, "eighthnote-icon", &dummyw, &dummyh);
@@ -328,6 +392,8 @@ void NedResource::initialize() {
 
 	loadPixmap(lyrics_icon, &m_lyrics_icon, "lyrics-icon", &dummyw, &dummyh);
 
+	loadPixmap(key_icon, &m_key_icon, "key-icon", &dummyw, &dummyh);
+
 	loadPixmap(normal_note_icon, &m_normal_note_icon, "normal-note-icon", &dummyw, &dummyh);
 	loadPixmap(drum1_icon, &m_drum1_icon, "drum1-icon", &dummyw, &dummyh);
 	loadPixmap(drum2_icon, &m_drum2_icon, "drum2-icon", &dummyw, &dummyh);
@@ -339,6 +405,7 @@ void NedResource::initialize() {
 	loadPixmap(dcross_icon, &m_dcross_icon, "dcross-icon", &dummyw, &dummyh);
 	loadPixmap(dflat_icon, &m_dflat_icon, "dflat-icon", &dummyw, &dummyh);
 	loadPixmap(stacc_icon, &m_stacc_icon, "stacc-icon", &dummyw, &dummyh);
+	loadPixmap(staccatissimo_icon, &m_staccatissimo_icon, "staccatissimo-icon", &dummyw, &dummyh);
 	loadPixmap(str_pizz_icon, &m_str_pizz_icon, "str-pizz-icon", &dummyw, &dummyh);
 	loadPixmap(tenuto_icon, &m_tenuto_icon, "tenuto-icon", &dummyw, &dummyh);
 	loadPixmap(sforzato_icon, &m_sforzato_icon, "sforzato-icon", &dummyw, &dummyh);
@@ -346,8 +413,20 @@ void NedResource::initialize() {
 	loadPixmap(bow_up_icon, &m_bow_up_icon, "bow-up-icon", &dummyw, &dummyh);
 	loadPixmap(bow_down_icon, &m_bow_down_icon, "bow-down-icon", &dummyw, &dummyh);
 
-	loadPixmap(brace_icon, &m_brace_icon, "brace_icon", &dummyw, &dummyh);
-	loadPixmap(bracket_icon, &m_bracket_icon, "bracket_icon", &dummyw, &dummyh);
+	loadPixmap(brace_icon, &m_brace_icon, "brace-icon", &dummyw, &dummyh);
+	loadPixmap(bracket_icon, &m_bracket_icon, "bracket-icon", &dummyw, &dummyh);
+
+	loadPixmap(ped_on_icon, &m_ped_on_icon, "ped-on-icon", &dummyw, &dummyh);
+	loadPixmap(ped_off_icon, &m_ped_on_icon, "ped-off-icon", &dummyw, &dummyh);
+	loadPixmap(trill_icon, &m_trill_icon, "trill-icon", &dummyw, &dummyh);
+	loadPixmap(prall_icon, &m_prall_icon, "prall-icon", &dummyw, &dummyh);
+	loadPixmap(open_icon, &m_open_icon, "open-icon", &dummyw, &dummyh);
+	loadPixmap(mordent_icon, &m_mordent_icon, "mordent-icon", &dummyw, &dummyh);
+	loadPixmap(fermata_icon, &m_fermata_icon, "fermata-icon", &dummyw, &dummyh);
+	loadPixmap(arpeggio_icon, &m_arpeggio_icon, "arpeggio-icon", &dummyw, &dummyh);
+
+	loadPixmap(lines, &m_lines, "lines", &m_lines_buf_width, &m_lines_buf_height);
+	loadPixmap(signs, &m_signs, "signs", &m_signs_buf_width, &m_signs_buf_height);
 
 	loadPixmap(arrow_up_icon, &m_arrow_up_icon, "my-arrow-up-icon", &dummyw, &dummyh);
 	loadPixmap(arrow_down_icon, &m_arrow_down_icon, "my-arrow-down-icon", &dummyw, &dummyh);
@@ -406,14 +485,14 @@ int NedResource::loadfontface(char *font_file_name) {
 	
 	
 	if (FT_Init_FreeType(&thelibrary) != 0) {
-		fprintf(stderr, "Error in FT_Init_FreeType\n");
+		DbgMsg(DBG_CRITICAL, "Error in FT_Init_FreeType\n");
 		return 0;
 	}
 	
 	openargs.flags = FT_OPEN_PATHNAME;
 	openargs.pathname = font_file_name;
 	if (FT_Open_Face( thelibrary, &openargs, 0, &theface) != 0) { 
-		fprintf(stderr, "Error in FT_Open_Face\n");
+		DbgMsg(DBG_CRITICAL, "Error in FT_Open_Face\n");
 		return 0;
 	}
 	
@@ -589,12 +668,18 @@ void NedResource::NoteOff(int channel, int pitch) {
 
 void NedResource::SeqReset() {
 	snd_seq_event_t ev;
+
+	if (m_sequ == NULL) return;
 	snd_seq_ev_clear(&ev);
 
+	for (int i = 0; i < 16; i++) {
+		MidiCtrl(MIDI_CTL_SUSTAIN, i, 1);
+	}
+
 	ev.type = SND_SEQ_EVENT_RESET;
 	ev.data.note.channel = 0;
 	ev.data.note.velocity = 0;
-	ev.data.note.note = 0;;
+	ev.data.note.note = 0;
 	snd_seq_ev_set_subs(&ev);  
 	snd_seq_ev_set_direct(&ev);
 	snd_seq_event_output_direct(m_sequ, &ev);
@@ -661,7 +746,13 @@ void NedResource::write_config() {
 	if (m_last_xml_dir  != NULL) {
 		fprintf(fp, "LASTXMLDIR = %s\n", m_last_xml_dir );
 	}
+	if (m_last_midi_dir != NULL) {
+		fprintf(fp, "LASTMIDIDIR = %s\n", m_last_midi_dir );
+	}
 	fprintf(fp, "PRINTCMD = %s\n", m_print_cmd);
+	gtk_window_get_size (GTK_WINDOW(m_main_window->getWindow()), &m_width, &m_height);
+	fprintf(fp, "WIDTH = %d\n", m_width?m_width:WIDTH);
+	fprintf(fp, "HEIGHT = %d\n", m_height?m_height:HEIGHT);
 	for (i = 0; i < MAX_RECENT_FILES; i++) {
 		if (m_recent_files[i] != NULL) {
 			fprintf(fp, "RECENTFILE %d = %s\n", i, m_recent_files[i]);
@@ -711,7 +802,7 @@ void NedResource::read_config() {
 	strcat(fname, "/");
 	strcat(fname, CONFIG_FILE);
 	if ((fp = fopen(fname, "r")) == NULL) {
-		fprintf(stderr, "cannot read config. Ignore this if this is the first call!\n");
+		DbgMsg(DBG_CRITICAL, "cannot read config. Ignore this if this is the first call!\n");
 		return;
 	}
 	m_input_line = 1;
@@ -761,6 +852,36 @@ void NedResource::read_config() {
 			}
 			m_print_cmd = strdup(buffer);
 		}
+		else if (!strcmp(buffer, "WIDTH")) {
+			if (!readWord(fp, buffer)) {
+				sprintf(Str, "cannot read config: line %d: '=' expected", m_input_line);
+				Warning(Str);
+				fclose(fp);
+				return;
+			}
+			if (!readInt(fp, &m_width)) {
+				sprintf(Str, "cannot read config: line %d: width size expected", m_input_line);
+				Warning(Str);
+				fclose(fp);
+				m_width = WIDTH;
+				return;
+			}
+		}
+		else if (!strcmp(buffer, "HEIGHT")) {
+			if (!readWord(fp, buffer)) {
+				sprintf(Str, "cannot read config: line %d: '=' expected", m_input_line);
+				Warning(Str);
+				fclose(fp);
+				return;
+			}
+			if (!readInt(fp, &m_height)) {
+				sprintf(Str, "cannot read config: line %d: height size expected", m_input_line);
+				Warning(Str);
+				fclose(fp);
+                m_height = HEIGHT;
+				return;
+			}
+		}
 		else if (!strcmp(buffer, "LASTFOLDER")) {
 			if (!readWord(fp, buffer)) {
 				sprintf(Str, "cannot read config: line %d: '=' expected", m_input_line);
@@ -791,6 +912,21 @@ void NedResource::read_config() {
 			}
 			m_last_xml_dir = strdup(buffer);
 		}
+		else if (!strcmp(buffer, "LASTMIDIDIR")) {
+			if (!readWord(fp, buffer)) {
+				sprintf(Str, "cannot read config: line %d: '=' expected", m_input_line);
+				Warning(Str);
+				fclose(fp);
+				return;
+			}
+			if (!readTillEnd(fp, buffer)) {
+				sprintf(Str, "cannot read config: line %d: last midi file expected", m_input_line);
+				Warning(Str);
+				fclose(fp);
+				return;
+			}
+			m_last_midi_dir = strdup(buffer);
+		}
 		else if (!strcmp(buffer, "RECENTFILE")) {
 			if (!readInt(fp, &idx) || idx < 0 || idx >= MAX_RECENT_FILES) {
 				sprintf(Str, "cannot read config: line %d: index of recent file expected", m_input_line);
@@ -843,11 +979,11 @@ bool NedResource::equalContent(GList *l1, GList *l2, NedChordOrRest *except, int
 	lendiff = g_list_length(l2) - g_list_length(l1);
 	if (lendiff != 0) {
 		if (except == NULL) {
-			printf("FAIL A\n"); fflush(stdout);
+			DbgMsg(DBG_TESTING, "FAIL A\n"); 
 			return FALSE;
 		}
 		if (lendiff != 1) {
-			printf("FAIL B\n"); fflush(stdout);
+			DbgMsg(DBG_TESTING, "FAIL B\n"); 
 			return FALSE;
 		}
 	}
@@ -856,17 +992,17 @@ bool NedResource::equalContent(GList *l1, GList *l2, NedChordOrRest *except, int
 	for (lptr2 = g_list_first(l2); lptr2; lptr2 = g_list_next(lptr2)) {
 		if (g_list_find(l1, lptr2->data) == NULL) {
 			if (except != NULL || *diff_pos >= 0) {
-				printf("FAIL C\n"); fflush(stdout);
+				DbgMsg(DBG_TESTING, "FAIL C\n"); 
 				return FALSE;
 			}
 			if (lptr2->data != except) {
-				printf("FAIL D\n"); fflush(stdout);
+				DbgMsg(DBG_TESTING, "FAIL D\n"); 
 				return FALSE;
 			}
 			*diff_pos =  g_list_position(l2, lptr2);
 		}
 	}
-	printf("IDENT\n"); fflush(stdout);
+	DbgMsg(DBG_TESTING, "IDENT\n"); 
 	return TRUE;
 }
             
@@ -882,6 +1018,15 @@ void NedResource::Abort(const char *s) {
 	exit(10);
 }
 
+#define CURRENT_DEBUG_LEVEL DBG_CRITICAL
+void NedResource::DbgMsg(int level, const char *format, ...) {
+	va_list ap;
+	if (level < CURRENT_DEBUG_LEVEL) return;
+	va_start (ap, format);
+	vprintf(format, ap);
+	va_end(ap);
+}
+
 void NedResource::Warning(const char *s) {
 	GtkWidget *error_dialog;
 	error_dialog = gtk_message_dialog_new (NULL, 
@@ -910,7 +1055,7 @@ void NedResource::loadPixmap(const guint8 *data, GdkPixbuf **pixbuf, const char
         GError *err = NULL;
         *pixbuf = gdk_pixbuf_new_from_inline(-1, data, false, &err);
         if (err != NULL) {
-                fprintf(stderr, "NedResource::loadPixmap: Fehler %d beim Laden einer Pixmap: %s\n", err->code, err->message);
+                DbgMsg(DBG_CRITICAL, "NedResource::loadPixmap: Fehler %d beim Laden einer Pixmap: %s\n", err->code, err->message);
                 exit(1);
         }
         *width = gdk_pixbuf_get_width(*pixbuf);
@@ -923,19 +1068,6 @@ void NedResource::loadPixmap(const guint8 *data, GdkPixbuf **pixbuf, const char
 }
 
 
-/* The function "sub_time(struct timeval *future, struct timeval *now)" */
-/* subtracts "now" from "future" and returns the result in msecs.       */
-
-unsigned long NedResource::sub_time(struct timeval *future, struct timeval *now) {
-        if (timercmp(future, now, <=)) return 0;
-	if (now->tv_usec > future->tv_usec) {
-		return (future->tv_sec - now->tv_sec - 1) * 1000 + (now->tv_usec - future->tv_usec) / 1000;
-	}
-        return (future->tv_sec - now->tv_sec) * 1000 + (future->tv_usec - now->tv_usec) / 1000;
-}
-
-
-
 void NedResource::unreadWord(char *word) {
 	strcpy(m_word_buffer, word);
 }
@@ -1201,7 +1333,7 @@ bool NedResource::readHex(FILE *fp, int *val) {
 	if (!not_eof) return FALSE;
 	if (!((buffer[0] >= 'A' && buffer[0] <= 'F') || (buffer[0] >= 'a' && buffer[0] <= 'f') ||
 		(buffer[0] >= '0' && buffer[0] <= '9'))) {
-		printf("raus bei 1 buffer[0] = %c (0x%x)\n", buffer[0],buffer[0]); fflush(stdout);
+		DbgMsg(DBG_TESTING, "raus bei 1 buffer[0] = %c (0x%x)\n", buffer[0],buffer[0]); 
 		return FALSE;
 	}
 		
@@ -1225,6 +1357,38 @@ bool NedResource::readHex(FILE *fp, int *val) {
 	return TRUE;
 }
 
+bool NedResource::readFloat(FILE *fp, double *val) {
+	int pos = 0;
+	char word[128];
+	bool dot_seen = false;
+
+	if (m_word_buffer[0] != '\0') {
+		strcpy(word, m_word_buffer);
+		m_word_buffer[0] = '\0';
+	}
+	else {
+		while (((word[0] = getc(fp)) != EOF) && (word[0] == ' ' || word[0] == '\t' || word[0] == '\n')) {
+			if (word[0] == '\n') {
+				NedResource::m_input_line++;
+			}
+		}
+		if (feof(fp)) return FALSE;
+		if (word[0] != '-' && word[0] != '+' && word[0] != '.' && word[0] < '0' && word[0] > '9') return false;
+		if (word[0] == '.') dot_seen = true;
+		pos++;
+		while ((word[pos] = getc(fp)) != EOF && ((word[pos] >= '0' && word[pos] <= '9') || word[pos] == '.')) {
+		 	if (word[pos] == '.') {
+				if (dot_seen) return false;
+				dot_seen = true;
+			}
+			pos++;
+		}
+		word[pos] = '\0';
+	}
+	if (sscanf(word, "%lf", val) != 1) return false;
+	return true;
+}
+
 bool NedResource::removeWhiteSpaces(char **s) {
 	gchar *cptr;
 	char buffer[4096];
@@ -1336,7 +1500,7 @@ void NedResource::subscribe(int idx) {
 }
 					
 
-void NedResource::prepareMidiEventList(NedMainWindow *main_window) {
+void NedResource::prepareMidiEventList(NedMainWindow *main_window, double tempo_inverse) {
 	GList *lptr1, *lptr2;
 
 	if (m_midi_events != NULL) {
@@ -1348,23 +1512,52 @@ void NedResource::prepareMidiEventList(NedMainWindow *main_window) {
 		}
 		g_list_free(m_midi_events);
 	}
+	for (lptr1 = g_list_first(m_fermata_list); lptr1; lptr1 = g_list_next(lptr1)) {
+		free(lptr1->data);
+	}
+	g_list_free(m_fermata_list);
+	m_fermata_list = NULL;
 	m_midi_events = NULL;
 	m_main_window = main_window;
 	m_system_start_time = 0;
+	m_tempo_inverse = tempo_inverse;
 }
 
 void NedResource::addMidiEvent(int channel, unsigned int offset, unsigned int type, int data1, int data2,
 			NedNote *note) {
 	MidiEventStruct *ev;
+	double current_tempo_inverse;
+	if ((ev = (MidiEventStruct *) g_malloc(sizeof(MidiEventStruct))) == NULL) {
+		Abort("error in g_malloc\n");
+	}
+	ev->channel = channel;
+	ev->midi_time = m_system_start_time + offset;
+	if (m_accel_midi_start < ev->midi_time && m_accel_midi_end >= ev->midi_time) {
+			current_tempo_inverse = m_accel_start_tempoinverse + 
+			(m_accel_end_tempoinverse - m_accel_start_tempoinverse) / 
+			(double) (m_accel_midi_end - m_accel_midi_start) *
+			(double) (ev->midi_time - m_accel_midi_start);
+			addTempoChange(offset, current_tempo_inverse);
+	}
+	ev->type = type;
+	ev->data1 = data1;
+	ev->data2 = data2;
+	ev->note = note;
+	ev->repetition = false;
+
+	placeMidiEvent(ev);
+}
+
+void NedResource::addMidiCtrlEvent(int channel, unsigned int offset, unsigned int ctrl_param, int val) {
+	MidiEventStruct *ev;
 	if ((ev = (MidiEventStruct *) g_malloc(sizeof(MidiEventStruct))) == NULL) {
 		Abort("error in g_malloc\n");
 	}
 	ev->channel = channel;
 	ev->midi_time = m_system_start_time + offset;
-	ev->type = type;
-	ev->data1 = data1;
-	ev->data2 = data2;
-	ev->note = note;
+	ev->type = SND_SEQ_EVENT_CONTROLLER;
+	ev->data1 = ctrl_param;
+	ev->data2 = val;
 
 	placeMidiEvent(ev);
 }
@@ -1383,15 +1576,66 @@ void NedResource::addMeasureEntry(NedMeasure *measure) {
 	placeMidiEvent(ev);
 }
 
+void NedResource::addTempoChange(unsigned int offset, double tempoinverse) {
+	MidiEventStruct *ev;
+
+	if ((ev = (MidiEventStruct *) g_malloc(sizeof(MidiEventStruct))) == NULL) {
+		Abort("error in g_malloc\n");
+	}
+	ev->channel = 0; /* dummy */
+	ev->midi_time = m_system_start_time + offset;
+	ev->type = PSEUDO_EVENT_TEMPO_CHANGE;
+	ev->tempoinverse = tempoinverse;
+	m_tempo_inverse = tempoinverse;
+
+	placeMidiEvent(ev);
+}
+
+void NedResource::addTempoChangeAbs(unsigned long long midi_time, double tempoinverse) {
+	MidiEventStruct *ev;
+
+	if ((ev = (MidiEventStruct *) g_malloc(sizeof(MidiEventStruct))) == NULL) {
+		Abort("error in g_malloc\n");
+	}
+	ev->channel = 0; /* dummy */
+	ev->midi_time = midi_time;
+	ev->type = PSEUDO_EVENT_TEMPO_CHANGE;
+	ev->tempoinverse = tempoinverse;
+
+	placeMidiEvent(ev);
+}
+
+void NedResource::setAccelRange(unsigned int offset, unsigned long long duration, double tempodiff) {
+	double tempo = 60000.0 / m_tempo_inverse;
+	m_accel_midi_start = m_system_start_time + offset;
+	m_accel_midi_end = m_accel_midi_start + duration;
+	m_accel_start_tempoinverse = m_tempo_inverse;
+	tempo += tempodiff;
+	m_accel_end_tempoinverse = 60000.0 / tempo;
+}
+
+void NedResource::addFermata(unsigned int offset) {
+	struct fermata_info *fermata_info;
+
+	if ((fermata_info = (struct fermata_info *) malloc(sizeof(struct fermata_info))) == NULL) {
+		Abort("NedResource::addFermata");
+	}
+	fermata_info->midi_time = m_system_start_time + offset;
+	fermata_info->tempo_inverse = m_tempo_inverse;
+	fermata_info->a_tempo = true;
+	m_fermata_list = g_list_append(m_fermata_list, fermata_info);
+}
+
 void NedResource::removeAllPseudosEvents() {
 	GList *lptr1, *lptr2, *lptr3;
 
+
 	lptr1 = g_list_first(m_midi_events);
 	while (lptr1) {
 		lptr2 = (GList *) (lptr1->data);
 		lptr3 = g_list_first(lptr2);
 		while (lptr3) {
-			if ((((MidiEventStruct *) lptr3->data)->type & PSEUDO_MASK) != 0) {
+			if ((((MidiEventStruct *) lptr3->data)->type & PSEUDO_FLAG) != 0) {
 				lptr2 = g_list_delete_link(lptr2, lptr3);
 				lptr3 = g_list_first(lptr2);
 			}
@@ -1410,6 +1654,54 @@ void NedResource::removeAllPseudosEvents() {
 	}
 }
 
+void NedResource::correctFermataTempo() {
+	GList *lptr1, *lptr2, *lptr3;
+	unsigned long long timenote, timeferm;
+	double target_tempo_inverse, current_tempo_inverse;
+	bool tempo_inserted;
+#define FERMATA_MIDI_BEFORE_DIST NOTE_4
+#define FERMATA_AFTER_MIDI_DIST NOTE_4
+#define FERMATA_MIDI_TEMPO_RATIO 3.0
+
+	if (m_fermata_list == NULL) return;
+
+	for (lptr1 = g_list_first(m_midi_events);  lptr1; lptr1 = g_list_next(lptr1)) {
+		lptr2 = (GList *) g_list_first((GList *) (lptr1->data));
+		tempo_inserted = false;
+		for (; !tempo_inserted && lptr2; lptr2 = g_list_next(lptr2)) {
+			if (((MidiEventStruct *) lptr2->data)->type == SND_SEQ_EVENT_NOTEON) {
+				timenote = ((MidiEventStruct *) lptr2->data)->midi_time;
+				for (lptr3 = g_list_first(m_fermata_list); lptr3; lptr3 = g_list_next(lptr3)) {
+					timeferm = ((struct fermata_info *) lptr3->data)->midi_time;
+					if (timeferm - FERMATA_MIDI_BEFORE_DIST <= timenote && timeferm > timenote) {
+						target_tempo_inverse = ((struct fermata_info *) lptr3->data)->tempo_inverse;
+						current_tempo_inverse = target_tempo_inverse + (FERMATA_MIDI_TEMPO_RATIO * target_tempo_inverse - target_tempo_inverse) /
+									(double) FERMATA_MIDI_BEFORE_DIST * (double) (timenote - (timeferm - FERMATA_MIDI_BEFORE_DIST));
+						addTempoChangeAbs(timenote, current_tempo_inverse);
+						tempo_inserted = true;
+						((struct fermata_info *) lptr3->data)->a_tempo = false;
+						break;
+					}
+					else if (timeferm <= timenote && timeferm + FERMATA_AFTER_MIDI_DIST > timenote) {
+						target_tempo_inverse = ((struct fermata_info *) lptr3->data)->tempo_inverse;
+						current_tempo_inverse = FERMATA_MIDI_TEMPO_RATIO * target_tempo_inverse + (target_tempo_inverse - FERMATA_MIDI_TEMPO_RATIO * target_tempo_inverse) /
+									(double) FERMATA_AFTER_MIDI_DIST * (double) (timenote - timeferm);
+						addTempoChangeAbs(timenote, current_tempo_inverse);
+						tempo_inserted = true;
+						((struct fermata_info *) lptr3->data)->a_tempo = false;
+						break;
+					}
+					else if (!(((struct fermata_info *) lptr3->data)->a_tempo) && timenote >= timeferm + FERMATA_AFTER_MIDI_DIST) {
+						addTempoChangeAbs(timenote, target_tempo_inverse);
+						((struct fermata_info *) lptr3->data)->a_tempo = true;
+					}
+				}
+			}
+		}
+	}
+}
+
+
 GList *NedResource::findListAtMeasure(int meas_number, NedMeasure **measure) {
 	GList *lptr1, *lptr2;
 
@@ -1473,13 +1765,13 @@ void NedResource::copyAllBetweenMeasures(int start_measure_number, int alternati
 					end_measure2->midi_end - end_measure2->midi_start;
 			extra_offs = midi_end_time - midi_start_time;
 			/*
-			printf("Sonderfall: end_measure->midi_end = %llu, start_measure->midi_start = %llu, extra_offs = %llu\n",
+			DbgMsg(DBG_TESTING, "Sonderfall: end_measure->midi_end = %llu, start_measure->midi_start = %llu, extra_offs = %llu\n",
 				end_measure2->midi_end / NOTE_4, start_measure->midi_start / NOTE_4,
-				 extra_offs / NOTE_4); fflush(stdout);
+				 extra_offs / NOTE_4); 
 				 */
 		}
 		else {
-			printf("start_measure_number = %d(%d) --> %llu, end_measure_number = %d(%d) --> %llu\n",
+			DbgMsg(DBG_CRITICAL, "start_measure_number = %d(%d) --> %llu, end_measure_number = %d(%d) --> %llu\n",
 				start_measure_number, start_measure->getMeasureNumber(),
 				start_measure->midi_start, end_measure_number,
 				end_measure->getMeasureNumber(), end_measure->midi_start); 
@@ -1495,8 +1787,8 @@ void NedResource::copyAllBetweenMeasures(int start_measure_number, int alternati
 #ifdef AAA
 	}
 #endif
-	//printf("end_measure->midi_start = %llu, start_measure->midi_start = %llu, extra_offs = %llu\n",
-	//end_measure->midi_start / NOTE_4, start_measure->midi_start / NOTE_4, extra_offs / NOTE_4); fflush(stdout);
+	//DbgMsg(DBG_TESTING, "end_measure->midi_start = %llu, start_measure->midi_start = %llu, extra_offs = %llu\n",
+	//end_measure->midi_start / NOTE_4, start_measure->midi_start / NOTE_4, extra_offs / NOTE_4); 
 	if (alternative1_measure_number >= 0) {
 		addTimeOffsetAfter(end_ptr, extra_offs - extra_offs2);
 	}
@@ -1514,9 +1806,9 @@ void NedResource::copyAllBetweenMeasures(int start_measure_number, int alternati
 	}
 	g_list_free(repeat_part);
 	/*
-	printf("nach clone:\n");
+	DbgMsg(DBG_TESTING, "nach clone:\n");
 	for (lptr1 = g_list_first(m_midi_events); lptr1; lptr1 = g_list_next(lptr1)) {
-		printf("###\n");
+		DbgMsg(DBG_TESTING, "###\n");
 		lptr2 = (GList *) g_list_first((GList *) (lptr1->data));
 		for(; lptr2; lptr2 = g_list_next(lptr2)) {
 			printMidiEvent((MidiEventStruct *) lptr2->data);
@@ -1584,7 +1876,7 @@ GList * NedResource::cloneAllMidiEventsBetween(GList *start_ptr, GList *end_ptr,
 	}
 
 	if (end_pos <= start_pos) {
-		printf("start_pos = %d, end_pos = %d\n", start_pos, end_pos); fflush(stdout);
+		DbgMsg(DBG_TESTING, "start_pos = %d, end_pos = %d\n", start_pos, end_pos); 
 		Abort("cloneAllMidiEventsBetween(3)");
 	}
 
@@ -1597,6 +1889,7 @@ GList * NedResource::cloneAllMidiEventsBetween(GList *start_ptr, GList *end_ptr,
 		}
 		*ev = *(((MidiEventStruct *) lptr2->data));
 		ev->midi_time += extra_offs;
+		ev->repetition = true;
 		repeat_part = g_list_append(repeat_part, ev);
 	}
 	for (start_ptr = g_list_next(start_ptr); start_ptr && (start_ptr != end_ptr); start_ptr = g_list_next(start_ptr)) {
@@ -1608,6 +1901,7 @@ GList * NedResource::cloneAllMidiEventsBetween(GList *start_ptr, GList *end_ptr,
 			}
 			*ev = *(((MidiEventStruct *) lptr2->data));
 			ev->midi_time += extra_offs;
+			ev->repetition = true;
 			repeat_part = g_list_append(repeat_part, ev);
 		}
 	}
@@ -1621,6 +1915,7 @@ GList * NedResource::cloneAllMidiEventsBetween(GList *start_ptr, GList *end_ptr,
 			}
 			*ev = *(((MidiEventStruct *) lptr2->data));
 			ev->midi_time += extra_offs;
+			ev->repetition = true;
 			repeat_part = g_list_append(repeat_part, ev);
 		}
 	}
@@ -1630,22 +1925,22 @@ GList * NedResource::cloneAllMidiEventsBetween(GList *start_ptr, GList *end_ptr,
 void NedResource::printMidiEvent(MidiEventStruct *ev) {
 	bool ok = true;
 	if (ev->type == PSEUDO_EVENT_MEASURE) return;
-	printf("type = ");
+	DbgMsg(DBG_TESTING, "type = ");
 	switch (ev->type) {
-		case SND_SEQ_EVENT_NOTEON: printf("SND_SEQ_EVENT_NOTEON"); break;
-		case SND_SEQ_EVENT_NOTEOFF: printf("SND_SEQ_EVENT_NOTEOFF"); break;
-		case PSEUDO_EVENT_MEASURE: printf("PSEUDO_EVENT_MEASURE"); ok = false; break;
-		default: printf("unknown %d(0x%x)", ev->type, ev->type);ok = false;  break;
+		case SND_SEQ_EVENT_NOTEON: DbgMsg(DBG_TESTING, "SND_SEQ_EVENT_NOTEON"); break;
+		case SND_SEQ_EVENT_NOTEOFF: DbgMsg(DBG_TESTING, "SND_SEQ_EVENT_NOTEOFF"); break;
+		case PSEUDO_EVENT_MEASURE: DbgMsg(DBG_TESTING, "PSEUDO_EVENT_MEASURE"); ok = false; break;
+		default: DbgMsg(DBG_TESTING, "unknown %d(0x%x)", ev->type, ev->type);ok = false;  break;
 	}
-	printf(", time = %llu", ev->midi_time / NOTE_4);
+	DbgMsg(DBG_TESTING, ", time = %llu", ev->midi_time / NOTE_4);
 	if (!ok) {
 		if (ev->type == PSEUDO_EVENT_MEASURE) {
-			printf(", measnum = %d\n", ev->measure->getMeasureNumber());
+			DbgMsg(DBG_TESTING, ", measnum = %d\n", ev->measure->getMeasureNumber());
 		}
-		putchar('\n'); fflush(stdout);
+		DbgMsg(DBG_TESTING, "\n"); 
 		return;
 	}
-	printf(", chan = %d, pitch = %d, vol = %d\n", ev->channel, ev->data1, ev->data2); fflush(stdout);
+	DbgMsg(DBG_TESTING, ", chan = %d, pitch = %d, vol = %d\n", ev->channel, ev->data1, ev->data2); 
 }
 
 void NedResource::placeMidiEvent(MidiEventStruct *ev) {
@@ -1675,7 +1970,19 @@ void NedResource::placeMidiEvent(MidiEventStruct *ev) {
 	m_midi_events = g_list_append(m_midi_events, event_list);
 }
 
-void NedResource::startReplay(NedNote *startNote) {
+void NedResource::collectTempoEvents(GList **tempo_chords) {
+	GList *lptr1, *lptr2;
+
+	for (lptr1 = g_list_first(m_midi_events); lptr1; lptr1 = g_list_next(lptr1)) {
+		lptr2 = (GList *) g_list_first((GList *) (lptr1->data));
+		for (; lptr2; lptr2 = g_list_next(lptr2)) {
+			if (((MidiEventStruct *) lptr2->data)->type != PSEUDO_EVENT_TEMPO_CHANGE) continue;
+			*tempo_chords = g_list_append(*tempo_chords, lptr2->data);
+		}
+	}
+}
+
+void NedResource::startReplay(NedNote *startNote, double tempo_inverse) {
 	GList *lptr1, *lptr2, *lptr22;
 	unsigned long long time_diff;
 	unsigned int sleeptime;
@@ -1683,6 +1990,7 @@ void NedResource::startReplay(NedNote *startNote) {
 	int num_notes = 0;
 	bool found;
 	bool first = true;
+	struct timeval now;
 
 	removeAllPseudosEvents();
 
@@ -1691,6 +1999,9 @@ void NedResource::startReplay(NedNote *startNote) {
 		return;
 	}
 
+	m_accel_midi_start = m_accel_midi_end = 0;
+	m_tempo_inverse = tempo_inverse;
+
 
 	m_play_ptr = g_list_first(m_midi_events);
 	if (startNote != NULL) {
@@ -1698,7 +2009,10 @@ void NedResource::startReplay(NedNote *startNote) {
 		for (m_play_ptr = g_list_first(m_midi_events); !found && m_play_ptr; m_play_ptr = g_list_next(m_play_ptr)) {
 			lptr2 = (GList *) g_list_first((GList *) (m_play_ptr->data));
 			for (; lptr2; lptr2 = g_list_next(lptr2)) {
-				if (((MidiEventStruct *) lptr2->data)->type == SND_SEQ_EVENT_NOTEON && ((MidiEventStruct *) lptr2->data)->note == startNote) {
+				if (((MidiEventStruct *) lptr2->data)->type == PSEUDO_EVENT_TEMPO_CHANGE) {
+					m_tempo_inverse = ((MidiEventStruct *) lptr2->data)->tempoinverse;
+				}
+				else if (((MidiEventStruct *) lptr2->data)->type == SND_SEQ_EVENT_NOTEON && ((MidiEventStruct *) lptr2->data)->note == startNote) {
 					found = true;
 					break;
 				}
@@ -1718,7 +2032,7 @@ void NedResource::startReplay(NedNote *startNote) {
 	   if (lptr1 != NULL) {
 		lptr22 = (GList *) (lptr1->data);
 		time_diff = ((MidiEventStruct *) lptr22->data)->midi_time - ((MidiEventStruct *) lptr2->data)->midi_time;
-		sleeptime = (unsigned int) ((double) time_diff / (double) NOTE_4 * m_main_window->getMidiTempoInverse());
+		sleeptime = (unsigned int) ((double) time_diff / (double) NOTE_4 * m_tempo_inverse);
 	   }
 	   for (; lptr2; lptr2 = g_list_next(lptr2)) {
 		switch (((MidiEventStruct *) lptr2->data)->type) {
@@ -1739,6 +2053,9 @@ void NedResource::startReplay(NedNote *startNote) {
 							((MidiEventStruct *) lptr2->data)->note->m_active = FALSE;
 							notes[num_notes++] = ((MidiEventStruct *) lptr2->data)->note;
 							break;
+			case SND_SEQ_EVENT_CONTROLLER:  MidiCtrl(((MidiEventStruct *) lptr2->data)->data1, ((MidiEventStruct *) lptr2->data)->channel,
+								((MidiEventStruct *) lptr2->data)->data2);
+							break;
 		}
 	   }
 	   m_play_ptr = lptr1;
@@ -1752,6 +2069,8 @@ void NedResource::startReplay(NedNote *startNote) {
 	m_main_window->repaintDuringReplay(notes, num_notes);
 
 	if (m_play_ptr) {
+		gettimeofday(&now, NULL);
+		addtime(&now, sleeptime, &m_expected_time);
 		m_handler_id = g_timeout_add (sleeptime, playNext, NULL);
 		//m_playing = TRUE; is already done
 	}
@@ -1761,6 +2080,23 @@ void NedResource::startReplay(NedNote *startNote) {
 	}
 }
 
+unsigned int NedResource::subtime(struct timeval *future, struct timeval *now) {
+	struct timeval t;
+	if (timercmp(future, now, >)) {
+		timersub(future, now, &t);
+		return t.tv_usec;
+	}
+	return 0;
+}
+
+void NedResource::addtime(struct timeval *now, unsigned int msecs, struct timeval *res) {
+	struct timeval t;
+
+	t.tv_sec = 0; 
+	t.tv_usec = msecs * 1000;
+	timeradd(now, &t, res);
+}
+
 gboolean NedResource::playNext(void *data) {
 	GList *lptr1, *lptr2, *lptr22;
 	unsigned int sleeptime;
@@ -1768,14 +2104,17 @@ gboolean NedResource::playNext(void *data) {
 	bool first = true;
 	NedNote *notes[1024];
 	int num_notes = 0;
+	struct timeval now;
 	g_source_remove (m_handler_id);
 	if (m_play_ptr == NULL) return FALSE;
 	if (!m_playing) {
 		if (m_midi_events != NULL) {
 			for (lptr1 = g_list_first(m_midi_events); lptr1; lptr1 = g_list_next(lptr1)) {
 				for (lptr2 = (GList *) g_list_first((GList *) (lptr1->data)); lptr2; lptr2 = g_list_next(lptr2)) {
-					NoteOff(((MidiEventStruct *) lptr2->data)->channel, ((MidiEventStruct *) lptr2->data)->data1);
-					((MidiEventStruct *) lptr2->data)->note->m_active = FALSE;
+					if (((MidiEventStruct *) lptr2->data)->type == SND_SEQ_EVENT_NOTEON) {
+						NoteOff(((MidiEventStruct *) lptr2->data)->channel, ((MidiEventStruct *) lptr2->data)->data1);
+						((MidiEventStruct *) lptr2->data)->note->m_active = FALSE;
+					}
 					
 				}
 			}
@@ -1785,13 +2124,15 @@ gboolean NedResource::playNext(void *data) {
 		return FALSE;
 	}
 
+
 	lptr2 = (GList *) g_list_first((GList *) (m_play_ptr->data));
 	lptr1 = g_list_next(m_play_ptr);
 	if (lptr1 != NULL) {
 		lptr22 = (GList *) (lptr1->data);
 		time_diff = ((MidiEventStruct *) lptr22->data)->midi_time - ((MidiEventStruct *) lptr2->data)->midi_time;
-		sleeptime = (unsigned int) ((double) time_diff / (double) NOTE_4 * m_main_window->getMidiTempoInverse());
 	}
+	gettimeofday(&now, NULL);
+	usleep(subtime(&m_expected_time, &now));
 	for (; lptr2; lptr2 = g_list_next(lptr2)) {
 		switch (((MidiEventStruct *) lptr2->data)->type) {
 			case SND_SEQ_EVENT_NOTEOFF: if (((MidiEventStruct *) lptr2->data)->note->getTieForward() == NULL) {
@@ -1817,8 +2158,16 @@ gboolean NedResource::playNext(void *data) {
 							((MidiEventStruct *) lptr2->data)->note->m_active = TRUE;
 							notes[num_notes++] = ((MidiEventStruct *) lptr2->data)->note;
 							break;
+			case SND_SEQ_EVENT_CONTROLLER:  MidiCtrl(((MidiEventStruct *) lptr2->data)->data1, ((MidiEventStruct *) lptr2->data)->channel,
+								((MidiEventStruct *) lptr2->data)->data2);
+							break;
+			case PSEUDO_EVENT_TEMPO_CHANGE: m_tempo_inverse = ((MidiEventStruct *) lptr2->data)->tempoinverse;
+							break;
 		}
 	}
+	sleeptime = (unsigned int) ((double) time_diff / (double) NOTE_4 * m_tempo_inverse);
+	gettimeofday(&now, NULL);
+	addtime(&now, sleeptime, &m_expected_time);
 	/*
 	if (num_notes > 0) {
 		m_main_window->setVisible(notes[0]);
@@ -1829,7 +2178,7 @@ gboolean NedResource::playNext(void *data) {
 	m_play_ptr = lptr1;
 
 	if (m_play_ptr) {
-		m_handler_id = g_timeout_add (sleeptime, playNext, NULL);
+		m_handler_id = g_timeout_add (sleeptime > 10 ? sleeptime - 10 : sleeptime, playNext, NULL);
 	}
 	else {
 		m_playing = FALSE;
@@ -1838,6 +2187,11 @@ gboolean NedResource::playNext(void *data) {
 	return FALSE;
 }
 
+void NedResource::stopReplay() {
+	m_playing = FALSE;
+	SeqReset();
+}
+
 void NedResource::playImmediately(int channel, int pgm, int pitch, int vol) {
 	snd_seq_event_t ev;
 
@@ -1929,7 +2283,7 @@ void *NedResource::getAdressOfMarker(int marker) {
 			return ((AddrStruct *) lptr->data)->addr;
 		}
 	}
-	printf("marker = %d\n", marker);
+	DbgMsg(DBG_CRITICAL, "marker = %d\n", marker);
 	Abort("NedResource::getAdressOfMarker");
 	return NULL;
 }
@@ -2043,7 +2397,7 @@ void NedResource::split_element(GList **elements, GList *sp_element, unsigned lo
 	while ((len2 = getPartLength(&dotcount)) > 0) {
 		pos++;
 		chord_or_rest2 = chord_or_rest1->cloneWithDifferentLength(len2, dotcount);
-		chord_or_rest_before->tieComplete(chord_or_rest2);
+		chord_or_rest_before->tieCompleteTo(chord_or_rest2);
 		chord_or_rest_before = chord_or_rest2;
 		*elements = g_list_insert(*elements, chord_or_rest2, pos);
 		delta1 -= len2;
@@ -2052,7 +2406,7 @@ void NedResource::split_element(GList **elements, GList *sp_element, unsigned lo
 	while ((len2 = getPartLength(&dotcount)) > 0) {
 		pos++;
 		chord_or_rest2 = chord_or_rest1->cloneWithDifferentLength(len2, dotcount);
-		chord_or_rest_before->tieComplete(chord_or_rest2);
+		chord_or_rest_before->tieCompleteTo(chord_or_rest2);
 		chord_or_rest_before = chord_or_rest2;
 		*elements = g_list_insert(*elements, chord_or_rest2, pos);
 		delta2 -= len2;
@@ -2065,8 +2419,46 @@ unsigned int NedResource::getPartLength(int *dotcount) {
 	*dotcount = 0;
 
 		
-	if (m_length_field_value >= WHOLE_NOTE) {
-		ret = WHOLE_NOTE;
+	if (m_length_field_value >= NOTE_2) {
+		ret = NOTE_2;
+	}
+	else {
+		for (m = NOTE_2; m >= NOTE_64; m >>= 1) {
+			if (m_length_field_value >= m * 3 / 2) {
+				*dotcount = 1;
+				ret = m;
+				break;
+			}
+			if (m_length_field_value >= m) {
+				ret = m;
+				break;
+			}
+		}
+	}
+	if (*dotcount == 1) {
+		m_length_field_value -= ret * 3 / 2;
+	}
+	else {
+		m_length_field_value -= ret;
+	}
+	return ret;
+}
+unsigned int NedResource::getPartLength(int *dotcount, bool is_rest, bool build_whole_rests, unsigned int measure_duration) {
+	unsigned int ret = 0, m;
+	*dotcount = 0;
+
+		
+	if (m_length_field_value >= NOTE_2) {
+		if (is_rest && build_whole_rests && m_length_field_value >= WHOLE_NOTE) {
+			m_length_field_value -= measure_duration;
+			return WHOLE_NOTE;
+		}
+		else if (!is_rest && m_length_field_value >= WHOLE_NOTE) {
+			ret = WHOLE_NOTE;
+		}
+		else {
+			ret = NOTE_2;
+		}
 	}
 	else {
 		for (m = NOTE_2; m >= NOTE_64; m >>= 1) {
@@ -2091,10 +2483,11 @@ unsigned int NedResource::getPartLength(int *dotcount) {
 }
 
 bool NedResource::fittingDuration(unsigned int duration) {
-	unsigned int m;
+	unsigned int m, d;
 
 	for (m = WHOLE_NOTE; m >= NOTE_64; m >>= 1) {
-		if (duration == m) return true;
+		d = duration / m;
+		if (m * d == duration) return true;
 	}
 	return false;
 }
@@ -2139,6 +2532,78 @@ bool NedResource::selection_has_uncomplete_tuplets(GList *elements) {
 	return FALSE;
 }
 
+int NedResource::determineLastLine(int treble_line, int clef) {
+	switch (clef) {
+		case BASS_CLEF: treble_line += 12; break;
+		case NEUTRAL_CLEF1:
+		case NEUTRAL_CLEF2: 
+		case ALTO_CLEF: treble_line += 6; break;
+		case TENOR_CLEF: treble_line += 8; break;
+		case SOPRAN_CLEF: treble_line += 2; break;
+	}
+	return treble_line;
+}
+
+const char *NedResource::getLilyPondClefName(int clef_number) {
+	switch (clef_number) {
+		case TREBLE_CLEF: return "treble";
+		case BASS_CLEF: return "bass";
+		case ALTO_CLEF: return "alto";
+		case SOPRAN_CLEF: return "soprano";
+		case TENOR_CLEF: return "tenor";
+		case NEUTRAL_CLEF1: 
+		case NEUTRAL_CLEF2: return "percussion";
+	}
+	return "treble";
+}
+const char *NedResource::getLilyPondKeySigName(int keysig_number) {
+	switch (keysig_number) {
+		case -6: return "\\key ges \\major";
+		case -5: return "\\key des \\major";
+		case -4: return "\\key as \\major";
+		case -3: return "\\key es \\major";
+		case -2: return "\\key bes \\major";
+		case -1: return "\\key f \\major";
+		case  1: return "\\key g \\major";
+		case  2: return "\\key d \\major";
+		case  3: return "\\key a \\major";
+		case  4: return "\\key e \\major";
+		case  5: return "\\key b \\major";
+		case  6: return "\\key fis \\major";
+	}
+	return "";
+}
+
+void NedResource::makeLilyString(char *ori, char *buffer) {
+	int len, i;
+	glong pos;
+	gchar* cpori;
+	gunichar cori;
+
+	strcpy(buffer+1, ori);
+	buffer[0] = '"';
+	cpori = buffer + 1;
+	cori = g_utf8_get_char(cpori);
+	while (cori != '\0') {
+		if (cori == '"') {
+			len = strlen(buffer);
+			pos = g_utf8_pointer_to_offset(buffer, cpori);
+			for (i = len + 1; i > pos; i--) {
+				buffer[i] = buffer[i - 1];
+			}
+			buffer[pos] = '\\';
+			cpori = g_utf8_next_char(cpori);
+		}
+		cpori = g_utf8_next_char(cpori);
+		cori = g_utf8_get_char(cpori);
+	}
+	len = strlen(buffer);
+	buffer[len++] = '"';
+	buffer[len++] = '\0';
+}
+			
+
+
 
 void NedResource::showLicense(GtkWindow *parent) {
 	const gchar *the_text = "This program is free software; you can redistribute it and/or modify it under the\n"
