@@ -30,6 +30,9 @@
 
 #define FONTFILE "nted/ntedfont.pfa"
 
+#define WIDTH 1023
+#define HEIGHT 767
+
 #define ZOOMLEVELS 15
 #define MAXELEMENTS  21
 #define DEFAULT_BORDER 0.2
@@ -72,7 +75,8 @@
 #define MIN_SELECTION_RECTANGLE_DIM 2.0
 
 #define TRIPLET_VAL_SHIFT_X 0.15
-#define TUPLET_XOVER 0.3
+#define TUPLET_XOVER 0.15
+#define TUPLET_XFRONTOVER 0.1
 #define TUPLET_MARKER_DIST 0.1
 #define TUPLET_MARKER_DIST_UP 0.2
 #define TUPLET_MARKER_DIST_DOWN 0.8
@@ -225,13 +229,23 @@
 #define STAT_SFORZANDO (1 <<  8)
 #define STAT_BOW_UP    (1 <<  9)
 #define STAT_BOW_DOWN  (1 << 10)
+#define STAT_PED_ON    (1 << 11)
+#define STAT_PED_OFF   (1 << 12)
+#define STAT_TRILL     (1 << 13)
+#define STAT_PRALL     (1 << 14)
+#define STAT_MORDENT   (1 << 15)
+#define STAT_OPEN      (1 << 16)
+#define STAT_STACCATISSIMO (1 << 17)
+#define STAT_FERMATA (1 << 18)
+#define STAT_ARPEGGIO (1 << 19)
 
 #define VIRTUAL_NATURAL_OFFS -111
 
-#define CLEF_SPACE 0.96
+#define CLEF_SPACE 0.9
 #define CLEF_POS 0.5
-#define TIME_SIGNATURE_SPACE 0.52
+#define TIME_SIGNATURE_SPACE 0.37
 #define SIG_X_SHIFT (0.87 * LINE_DIST)
+#define EXTRA_KEY_OFF 0.05
 #define MEAS_BAR_SPACE (0.74 * LINE_DIST)
 #define RIGHT_DIST (0.174 * LINE_DIST)
 //#define EXTRA_SPACE (0.652 * LINE_DIST)
@@ -321,6 +335,10 @@
 
 #define MAX_OVER_UNDER_LINES 12
 
+#define DBG_NO 0
+#define DBG_TESTING 1
+#define DBG_CRITICAL 2
+
 class NedNote;
 class NedChordOrRest;
 class NedMainWindow;
@@ -346,12 +364,16 @@ struct MidiEventStruct {
 	unsigned long long midi_time;
 	unsigned int type;
 #define PSEUDO_FLAG (0x1 << 17)
+#define PSEUDO2_FLAG (0x1 << 18)
 #define PSEUDO_MASK (0xffff << 16)
 #define NOTE_MASK (0xffff)
 #define PSEUDO_EVENT_MEASURE (PSEUDO_FLAG | 0x1)
+#define PSEUDO_EVENT_TEMPO_CHANGE (PSEUDO2_FLAG | 0x2)
 	int data1, data2;
 	NedNote *note;
 	NedMeasure *measure;
+	double tempoinverse;
+	bool repetition;
 };
 
 struct paper_info_struct {
@@ -368,11 +390,13 @@ class NedResource {
 	public:
 		static void initialize();
 		static void cleanup();
+		static void setWindow(NedMainWindow *ned_main_window) {m_main_window = ned_main_window;}
 		static char *getFontFileName() {return m_font_file_name;}
 		static cairo_scaled_font_t *getScaledFont(int zoom_level);
 		static double getStartZoomScale() {return m_zoom_factors[START_ZOOM_LEVEL];}
 		static double getZoomFactor(int zoom_level);
 		static void Abort(const char *s);
+		static void DbgMsg(int level, const char *format, ...);
 		static void Warning(const char *s);
 		static void Info(const char *s);
 		static cairo_text_extents_t fontextentions[ZOOMLEVELS][MAXELEMENTS];
@@ -383,10 +407,9 @@ class NedResource {
 		static cairo_matrix_t *getFontMatrix(int zoom_level);
 		static cairo_font_options_t *getFontOptions() {return m_fontoptions;}
 #endif
-		/* The function "sub_time(struct timeval *future, struct timeval *now)" */
-		/* subtracts "now" from "future" and returns the result in msecs.       */
 
-		static unsigned long sub_time(struct timeval *future, struct timeval *now);
+		static GdkPixbuf *m_the_nted_icon;
+
 		static GdkPixbuf *m_32ndnote_icon;
 		static GdkPixbuf *m_64thnote_icon;
 		static GdkPixbuf *m_eightnote_icon;
@@ -424,6 +447,7 @@ class NedResource {
 		static GdkPixbuf *m_color_icon;
 
 		static GdkPixbuf *m_lyrics_icon;
+		static GdkPixbuf *m_key_icon;
 
 		static GdkPixbuf *m_normal_note_icon;
 		static GdkPixbuf *m_drum1_icon;
@@ -436,6 +460,7 @@ class NedResource {
 		static GdkPixbuf *m_dcross_icon;
 		static GdkPixbuf *m_dflat_icon;
 		static GdkPixbuf *m_stacc_icon;
+		static GdkPixbuf *m_staccatissimo_icon;
 		static GdkPixbuf *m_str_pizz_icon;
 		static GdkPixbuf *m_tenuto_icon;
 		static GdkPixbuf *m_sforzato_icon;
@@ -446,6 +471,22 @@ class NedResource {
 		static GdkPixbuf *m_brace_icon;
 		static GdkPixbuf *m_bracket_icon;
 
+		static GdkPixbuf *m_ped_on_icon;
+		static GdkPixbuf *m_ped_off_icon;
+		static GdkPixbuf *m_trill_icon;
+		static GdkPixbuf *m_prall_icon;
+		static GdkPixbuf *m_open_icon;
+		static GdkPixbuf *m_mordent_icon;
+		static GdkPixbuf *m_fermata_icon;
+		static GdkPixbuf *m_arpeggio_icon;
+
+		static GdkPixbuf *m_lines;
+		static int m_lines_buf_width, m_lines_buf_height;
+
+		static GdkPixbuf *m_signs;
+		static int m_signs_buf_width, m_signs_buf_height;
+
+
 		static NedBbox m_acc_bbox;
 		static double getFlagWidth() {return m_flag_width;}
 		static void unreadWord(char *word);
@@ -458,6 +499,7 @@ class NedResource {
 		static bool readUnsignedInt(FILE *fp, unsigned int *val);
 		static bool readLong(FILE *fp, unsigned long long*val);
 		static bool readHex(FILE *fp, int *val);
+		static bool readFloat(FILE *fp, double *val);
 		static bool removeWhiteSpaces(char **s);
 		static const char *m_error_message;
 		static int m_input_line;
@@ -467,15 +509,24 @@ class NedResource {
 		static void NoteOn(int channel, int pitch, int vol);
 		static void NoteOff(int channel, int pitch);
 		static void SeqReset();
+
+		static void setAccelRange(unsigned int offset, unsigned long long duration, double tempodiff);
 		static void addMidiEvent(int channel, unsigned int offset, unsigned int type, int data1, int data2,
 			NedNote *note);
+		static void addMidiCtrlEvent(int channel, unsigned int offset, unsigned int ctrl_param, int val);
+		static void addTempoChange(unsigned int offset, double tempoinverse);
+		static void addTempoChangeAbs(unsigned long long midi_time, double tempoinverse);
 		static void addMeasureEntry(NedMeasure *measure);
-		static void prepareMidiEventList(NedMainWindow *main_window);
-		static void startReplay(NedNote *startNote);
+		static void addTempoChange(double tempoinverse);
+		static void addFermata(unsigned int offset);
+		static void correctFermataTempo();
+		static void prepareMidiEventList(NedMainWindow *main_window, double tempo_inverse);
+		static void startReplay(NedNote *startNote, double tempo_inverse);
+		static void collectTempoEvents(GList **tempo_chords);
 		static void copyAllBetweenMeasures(int start_measure_number, int alternative1_measure_number, int end_measure_number);
 		static bool isPlaying() {return m_playing;}
 		static bool m_midi_echo;
-		static void stopReplay() {m_playing = FALSE;}
+		static void stopReplay();
 		static void increaseSystemStartTime(unsigned long long system_duration);
 		static const char *GM_Instruments[];
 		static int getNumInstruments();
@@ -495,10 +546,13 @@ class NedResource {
 		static bool m_avoid_immadiate_play;
 		static struct paper_info_struct *getPaperInfo(const char *papername);
 		static const char *m_print_cmd;
+		static int m_width;
+		static int m_height;
 		static int m_tuplet_tab[14];
 		static void appendTempFileName(char *fname);
-		static char *m_last_folder;
+		static const char *m_last_folder;
 		static char *m_last_xml_dir;
+		static char *m_last_midi_dir;
 		static GList *clone_chords_and_rests(GList *group, bool remove_completely_hidden_voices);
 		static void split_element(GList **elements, GList *sp_element, unsigned long long delta1, unsigned long long delta2);
 		static bool test_for_incomplete_tuplets(GList *elements);
@@ -506,8 +560,16 @@ class NedResource {
 		static bool selection_has_uncomplete_tuplets(GList *elements);
 		static void setLengthField(unsigned int length_field_value) {m_length_field_value = length_field_value;}
 		static unsigned int getPartLength(int *dotcount);
+		static unsigned int getPartLength(int *dotcount, bool is_rest, bool build_whole_rests, unsigned int measure_duration);
 		static bool fittingDuration(unsigned int duration);
 		static bool fittingPosition(unsigned long long position);
+		static unsigned int subtime(struct timeval *future, struct timeval *now);
+		static void addtime(struct timeval *now, unsigned int msecs, struct timeval *res);
+		static unsigned long long getSystemStartTime() {return m_system_start_time;}
+		static int determineLastLine(int treble_line, int clef);
+		static const char *getLilyPondClefName(int clef_number);
+		static const char *getLilyPondKeySigName(int keysig_number);
+		static void makeLilyString(char *ori, char *buffer);
 	private:
 		static void printMidiEvent(MidiEventStruct *ev);
 		static void removeAllPseudosEvents();
@@ -550,12 +612,19 @@ class NedResource {
 		static int loadfontface(char *font_file_name);
 		static void loadPixmap(const guint8 *data, GdkPixbuf **pixbuf, const char *stock_id, int *width, int *height);
 		static GtkIconFactory *m_icon_factory;
+		static struct timeval m_expected_time;
 		static double m_flag_width;
 		static char m_word_buffer[128];
 		static snd_seq_t *m_sequ;
 		static int m_last_immediate_pitch;
 		static int m_last_immediate_channel;
 		static GList *m_temp_file_names;
+		static double m_tempo_inverse;
+		static unsigned long long m_accel_midi_start;
+		static unsigned long long m_accel_midi_end;
+		static double m_accel_start_tempoinverse;
+		static double m_accel_end_tempoinverse;
+		static GList *m_fermata_list;
 };
 
 
