@@ -48,7 +48,7 @@ const char NedStaff::m_flatPos[7][7] = {
 
 NedStaff::NedStaff(NedSystem *system, double ypos, double width, int nr, bool start) :
 m_staffelems(NULL), m_ypos(ypos), m_width(width), m_top_y_border(0), m_lyrics_lines(0), m_bottom_y_border(4 * LINE_DIST), m_staff_number(nr), m_clef_type(TREBLE_CLEF),
-m_clef_octave_shift(0), m_keysig(0), m_system(system), 
+m_clef_octave_shift(0), m_keysig(0), m_volume_changes(NULL), m_system(system),
 m_real_multiple_voices(FALSE) {
 	int i;
 
@@ -81,15 +81,24 @@ void NedStaff::appendWholeRest(NedCommandList *command_list /* = NULL */) {
 	}
 }
 
-void NedStaff::appendAppropriateWholes() {
+void NedStaff::setUpbeatRests(unsigned int upbeat_inverse) {
 	int i;
 
 	for (i = 0; i < VOICE_COUNT; i++) {
-		 m_voices[i]->appendAppropriateWholes();
+		 m_voices[i]->setUpbeatRests(upbeat_inverse);
 	}
 }
 
-void NedStaff::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array) {
+
+void NedStaff::appendAppropriateWholes(int start_measure) {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		 m_voices[i]->appendAppropriateWholes(start_measure);
+	}
+}
+
+void NedStaff::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, bool first) {
 	GList *lptr;
 	m_clef_type = clef_and_key_array[3 * m_staff_number + 0];
 	m_clef_octave_shift = clef_and_key_array[3 * m_staff_number + 1];
@@ -109,12 +118,27 @@ void NedStaff::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array) {
 		}
 	}
 	k = (m_keysig < 0) ? -m_keysig : m_keysig;
-	staff_start = CLEF_SPACE + TIME_SIGNATURE_SPACE + k * SIG_X_SHIFT;
+	staff_start = CLEF_SPACE + k * SIG_X_SHIFT + 2 * EXTRA_KEY_OFF;
+	if (first) {
+		staff_start += TIME_SIGNATURE_SPACE;
+	}
 	if (m_system->m_system_start < staff_start) {
 		m_system->m_system_start = staff_start;
 	}
 }
 
+void NedStaff::determineVolume(unsigned int *vol, NedChordOrRest *element, int staff_nr) {
+	m_voices[0]->determineVolume(vol, element, staff_nr);
+}
+
+void NedStaff::determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found) {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		 m_voices[i]->determineTempoInverse(element, till, tempoinverse, found);
+	}
+}
+
 GList *NedStaff::getFirstChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required) {
 	if (voice_nr < 0 || voice_nr >= VOICE_COUNT) {
 		NedResource::Abort("NedStaff::getFirstChordOrRest");
@@ -242,6 +266,27 @@ void NedStaff::findSelectedStaffs(double yp, NedBbox *sel_rect, int *number_of_f
 NedMainWindow *NedStaff::getMainWindow() {return m_system->getPage()->getMainWindow();}
 NedPage *NedStaff::getPage() {return m_system->getPage();}
 
+double NedStaff::getSystemEnd() {
+	return getPage()->getContentWidth() + TIE_X_RIGHT_OVER;
+}
+
+double NedStaff::getSystemStart() {
+	return getSystem()->m_system_start - TIE_X_OFFS;
+}
+
+GList *NedStaff::getStaffElements(unsigned long long midi_time) {
+	GList *lptr, *elems = NULL;
+
+	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
+		if (((NedChordOrRest *) lptr->data)->getMidiTime() == midi_time) {
+			elems = g_list_append(elems, lptr->data);
+		}
+	}
+
+	return elems;
+}
+
+
 void NedStaff::draw(cairo_t *cr, double botpos, bool first_page, double indent) {
 	GList *lptr;
 	double leftx = getMainWindow()->getLeftX();
@@ -267,7 +312,7 @@ void NedStaff::draw(cairo_t *cr, double botpos, bool first_page, double indent)
 #ifdef DRAW_TOP_BOTTOM_BORDERS
 	cairo_new_path(cr);
 	cairo_set_source_rgb (cr, 1.0, 0.0, 0.0);
-	//printf("(4)topy ist %f == %d, boty ist %f == %d\n", m_top_y_border, (int) ( m_top_y_border / LINE_DIST), m_bottom_y_border,  (int) (m_bottom_y_border / LINE_DIST));
+	//NedResource::DbgMsg(DBG_TESTING, "(4)topy ist %f == %d, boty ist %f == %d\n", m_top_y_border, (int) ( m_top_y_border / LINE_DIST), m_bottom_y_border,  (int) (m_bottom_y_border / LINE_DIST));
 	cairo_move_to(cr, X_POS_PAGE_REL(0), Y_POS_SYS_REL(m_ypos + m_top_y_border));
 	cairo_line_to(cr, X_POS_PAGE_REL(m_width), Y_POS_SYS_REL(m_ypos + m_top_y_border));
 	cairo_stroke(cr);
@@ -363,41 +408,43 @@ void NedStaff::draw(cairo_t *cr, double botpos, bool first_page, double indent)
 		}
 		cairo_stroke(cr);
 	}
-	int n = getMainWindow()->getNumerator(getSystem()->getNumberOfFirstMeasure());
-	if (n > 9) {
-		glyph[0].index = NUMBERBASE + n / 10;
-		glyph[0].x =  X_POS_PAGE_REL(sig_xpos - SIG_HALF);
-		glyph[0].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
-		glyph[1].index = NUMBERBASE + n % 10;
-		glyph[1].x =  X_POS_PAGE_REL(sig_xpos + SIG_HALF);
-		glyph[1].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
-		cairo_show_glyphs(cr, glyph, 2);
+	if (first_page || getMainWindow()->isTimsigChangingMeasure(getSystem()->getNumberOfFirstMeasure())) {
+		int n = getMainWindow()->getNumerator(getSystem()->getNumberOfFirstMeasure());
+		if (n > 9) {
+			glyph[0].index = NUMBERBASE + n / 10;
+			glyph[0].x =  X_POS_PAGE_REL(sig_xpos - SIG_HALF);
+			glyph[0].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
+			glyph[1].index = NUMBERBASE + n % 10;
+			glyph[1].x =  X_POS_PAGE_REL(sig_xpos + SIG_HALF);
+			glyph[1].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
+			cairo_show_glyphs(cr, glyph, 2);
+		}
+		else {
+	
+			glyph[0].index = NUMBERBASE + n;
+			glyph[0].x =  X_POS_PAGE_REL(sig_xpos);
+			glyph[0].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
+			cairo_show_glyphs(cr, glyph, 1);
+		}
+		n = getMainWindow()->getDenominator(getSystem()->getNumberOfFirstMeasure());
+		if (n > 9) {
+			glyph[0].index = NUMBERBASE + n / 10;
+			glyph[0].x =  X_POS_PAGE_REL(sig_xpos - SIG_HALF);
+			glyph[0].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
+			glyph[1].index = NUMBERBASE + n % 10;
+			glyph[1].x =  X_POS_PAGE_REL(sig_xpos + SIG_HALF);
+			glyph[1].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
+			cairo_show_glyphs(cr, glyph, 2);
+		}
+		else {
+			glyph[0].index = NUMBERBASE + n;
+			glyph[0].x =  X_POS_PAGE_REL(sig_xpos);
+			glyph[0].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
+			cairo_show_glyphs(cr, glyph, 1);
+		}
+		cairo_stroke(cr);
 	}
-	else {
-
-		glyph[0].index = NUMBERBASE + n;
-		glyph[0].x =  X_POS_PAGE_REL(sig_xpos);
-		glyph[0].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
-		cairo_show_glyphs(cr, glyph, 1);
-	}
-	n = getMainWindow()->getDenominator(getSystem()->getNumberOfFirstMeasure());
-	if (n > 9) {
-		glyph[0].index = NUMBERBASE + n / 10;
-		glyph[0].x =  X_POS_PAGE_REL(sig_xpos - SIG_HALF);
-		glyph[0].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
-		glyph[1].index = NUMBERBASE + n % 10;
-		glyph[1].x =  X_POS_PAGE_REL(sig_xpos + SIG_HALF);
-		glyph[1].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
-		cairo_show_glyphs(cr, glyph, 2);
-	}
-	else {
-		glyph[0].index = NUMBERBASE + getMainWindow()->getDenominator(getSystem()->getNumberOfFirstMeasure());
-		glyph[0].x =  X_POS_PAGE_REL(sig_xpos);
-		glyph[0].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
-		cairo_show_glyphs(cr, glyph, 1);
-	}
-	cairo_stroke(cr);
-
+	
 	for (i = 0; i < VOICE_COUNT; i++) {
 		m_voices[i]->draw(cr);
 	}
@@ -455,6 +502,13 @@ double NedStaff::getHeight() {
 	return 4 * LINE_DIST;
 }
 
+double NedStaff::getRealYPosOfLine(int line) {
+	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
+	double topy = getMainWindow()->getTopY();
+
+	return Y_POS_SYS_REL(m_ypos + 4 * LINE_DIST + line * LINE_DIST / 2.0);
+}
+
 void NedStaff::empty() {
 	int i;
 	for (i = 0; i < VOICE_COUNT; i++) {
@@ -462,23 +516,92 @@ void NedStaff::empty() {
 	}
 }
 
+bool NedStaff::hasOnlyRests() {
+	int i;
+	for (i = 0; i < VOICE_COUNT; i++) {
+		if (!m_voices[i]->hasOnlyRests()) return false;
+	}
+	return true;
+}
 
-bool NedStaff::trySelect(double x, double y) {
+
+
+void NedStaff::cutEmptyVoices() {
+	int i, j;
+	bool has_only_rests[VOICE_COUNT];
+	NedVoice *tmp_voice;
+	bool tmp_b;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		has_only_rests[i] = m_voices[i]->hasOnlyRests();
+	}
+	for (i = 0; i < VOICE_COUNT; i++) {
+		if (has_only_rests[i]) {
+			m_voices[i]->empty();
+			m_voices[i]->appendAppropriateWholes(0);
+		}
+	}
+		
+	for (i = 0; i < VOICE_COUNT - 1; i++) {
+		if (has_only_rests[i]) {
+			for (j = i + 1; j < VOICE_COUNT; j++) {
+				if (!has_only_rests[j]) {
+					tmp_voice = m_voices[i];
+					m_voices[i] = m_voices[j];
+					m_voices[j] = tmp_voice;
+					tmp_b = has_only_rests[i];
+					has_only_rests[i] = has_only_rests[j];
+					has_only_rests[j] = tmp_b;
+					break;
+				}
+			}
+		}
+	}
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->setVoiceNumber(i);
+	}
+}
+
+void NedStaff::testTies() {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->testTies();
+	}
+}
+
+bool NedStaff::trySelect(double x, double y, bool only_free_placeables) {
 	int i;
 	GList *lptr;
-	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
-		if (((NedChordOrRest *) lptr->data)->trySelect(x, y)) {
+	for (lptr = g_list_first(m_staffelems); !only_free_placeables && lptr; lptr = g_list_next(lptr)) {
+		if (((NedChordOrRest *) lptr->data)->trySelect(x, y, false)) {
 			return TRUE;
 		}
 	}
 	for (i = 0; i < VOICE_COUNT; i++) {
-		if (m_voices[i]->trySelect(x, y)) {
+		if (m_voices[i]->trySelect(x, y, only_free_placeables)) {
 			return TRUE;
 		}
 	}
 	return FALSE;
 }
 
+NedChordOrRest *NedStaff::findNearestElement(double x, double y, NedStaff *oldstaff, double *mindist, double *ydist) {
+	NedChordOrRest *nearestELement = NULL, *element;
+#define HYSTERESIS 0.4
+#ifdef ALL_NEAREST
+	for (i = 0; i < VOICE_COUNT; i++) {
+		if ((element = m_voices[i]->findNearestElement(x, y, mindist, ydist)) != NULL) {
+			nearestELement = element;
+		}
+	}
+#endif
+	if ((element = m_voices[0]->findNearestElement(x, y, (oldstaff == this) ? HYSTERESIS : 1.0, mindist, ydist)) != NULL) {
+		nearestELement = element;
+	}
+	return nearestELement;
+}
+
 bool NedStaff::tryChangeLength(NedChordOrRest *chord_or_rest) {
 	int i;
 	for (i = 0; i < VOICE_COUNT; i++) {
@@ -526,26 +649,27 @@ void NedStaff::handleStaffElements() {
 	}
 }
 
-void NedStaff::appendElementsOfMeasureLength(part *part_ptr, unsigned int meas_duration) {
+void NedStaff::appendElementsOfMeasureLength(part *part_ptr, unsigned int meas_duration, int meas_num) {
 	int i;
 	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->appendElementsOfMeasureLength(part_ptr, meas_duration);
+		m_voices[i]->appendElementsOfMeasureLength(part_ptr, meas_duration, meas_num);
 	}
 }
 
-bool NedStaff::tryInsertOrErease(double x, double y) {
-	return m_voices[getMainWindow()->getCurrentVoice()]->tryInsertOrErease(x, y);
+bool NedStaff::tryInsertOrErease(double x, double y, NedChordOrRest **newObj, bool force_rest) {
+	return m_voices[getMainWindow()->getCurrentVoice()]->tryInsertOrErease(x, y, newObj, force_rest);
 }
 
 
-bool NedStaff::findLine(double x, double y, int *ypos, int *line, int *bottom) {
+bool NedStaff::findLine(double x, double y, double *ypos, int *line, double *bottom) {
 	double topy = getMainWindow()->getTopY();
 	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
 	double yl = Y_POS_INVERS_STAFF_REL(y);
-	*line = (int) ((yl) / (LINE_DIST/2.0));
+	double yoffs = (yl >= 0.0) ? LINE_DIST / 4.0 : -LINE_DIST / 4.0;
+	*line = (int) ((yl + yoffs) / (LINE_DIST/2.0));
 	if (*line < -(MAX_OVER_UNDER_LINES + 8) || *line > MAX_OVER_UNDER_LINES) return FALSE;
-	*ypos = (int) (Y_POS_STAFF_REL(*line * LINE_DIST/2.0));
-	*bottom = (int) Y_POS_STAFF_REL(0);
+	*ypos = (Y_POS_STAFF_REL(*line * LINE_DIST/2.0) + yoffs);
+	*bottom =  Y_POS_STAFF_REL(0);
 	return TRUE;
 }
 
@@ -580,14 +704,14 @@ double NedStaff::placeStaff(double staffpos) {
 #define STAFF_MIN_DIST 2.0
 #ifdef PLACE_OUT_DEBUG
 	if (getSystem()->getSystemNumber()  < 2) {
-	printf("system %d staff %d: (1)staffpos = %f(%d)\n", getSystem()->getSystemNumber(), m_staff_number, staffpos, (int) (staffpos / (LINE_DIST / 2)));
+	NedResource::DbgMsg(DBG_TESTING, "system %d staff %d: (1)staffpos = %f(%d)\n", getSystem()->getSystemNumber(), m_staff_number, staffpos, (int) (staffpos / (LINE_DIST / 2)));
 	}
 #endif
 	if (m_top_y_border < 0) {
 #ifdef PLACE_OUT_DEBUG2
 		if (getSystem()->getSystemNumber()  == 0 && m_staff_number == 0) {
-			printf("erhoehe von %f auf %f\n", staffpos, staffpos -m_top_y_border);
-			fflush(stdout);
+			NedResource::DbgMsg(DBG_TESTING, "erhoehe von %f auf %f\n", staffpos, staffpos -m_top_y_border);
+			
 		}
 #endif
 		staffpos += -m_top_y_border;
@@ -595,7 +719,7 @@ double NedStaff::placeStaff(double staffpos) {
 	m_ypos = staffpos;
 #ifdef PLACE_OUT_DEBUG
 	if (getSystem()->getSystemNumber()  < 2) {
-	printf("(2)m_bottom_y_border = %f(%d), STAFF_MIN_DIST = %f(%d)\n", m_bottom_y_border, (int) (m_bottom_y_border / (LINE_DIST / 2)), STAFF_MIN_DIST, (int) (STAFF_MIN_DIST / (LINE_DIST / 2)));
+	NedResource::DbgMsg(DBG_TESTING, "(2)m_bottom_y_border = %f(%d), STAFF_MIN_DIST = %f(%d)\n", m_bottom_y_border, (int) (m_bottom_y_border / (LINE_DIST / 2)), STAFF_MIN_DIST, (int) (STAFF_MIN_DIST / (LINE_DIST / 2)));
 	}
 #endif
 	double yy = (m_lyrics_lines == 0) ? m_bottom_y_border : (m_bottom_y_border + 5 * LINE_DIST +  TEXT_Y_OFFS + m_lyrics_lines * LYRICS_HEIGHT);
@@ -607,7 +731,7 @@ double NedStaff::placeStaff(double staffpos) {
 	}
 #ifdef PLACE_OUT_DEBUG
 	if (getSystem()->getSystemNumber()  < 2) {
-	printf("(3)staffpos = %f(%d)\n", staffpos, (int) (staffpos / (LINE_DIST / 2)));
+	NedResource::DbgMsg(DBG_TESTING, "(3)staffpos = %f(%d)\n", staffpos, (int) (staffpos / (LINE_DIST / 2)));
 	}
 #endif
 
@@ -669,8 +793,8 @@ void NedStaff::deleteStaffElements(bool unexecute, GList *elems, unsigned long l
 		if (!unexecute) {
 			measure = m_system->getMeasure(((NedChordOrRest *) lptr2->data)->getMidiTime());
 			if (measure == NULL) {
-				printf("measure = 0 fuer time %llu (%llu)\n", ((NedChordOrRest *) lptr2->data)->getMidiTime(),
-					((NedChordOrRest *) lptr2->data)->getMidiTime() / NOTE_4); fflush(stdout);
+				NedResource::DbgMsg(DBG_TESTING, "measure = 0 fuer time %llu (%llu)\n", ((NedChordOrRest *) lptr2->data)->getMidiTime(),
+					((NedChordOrRest *) lptr2->data)->getMidiTime() / NOTE_4); 
 			}
 			((NedChordOrRest *) lptr2->data)->decrMidiTime(measure->midi_start);
 		}
@@ -782,11 +906,44 @@ void NedStaff::getCurrentClefAndKeysig(unsigned long long miditime, int *clef, i
 	}
 }
 
+int NedStaff::computeVolume(unsigned long long midi_time) {
+	GList *lptr;
+	VolumeChange *vol_change;
+	int volume = getMainWindow()->m_staff_contexts[m_staff_number].m_current_midi_volume;
+
+	for (lptr = g_list_first(m_volume_changes); lptr; lptr = g_list_next(lptr)) {
+		vol_change = (VolumeChange *) lptr->data;
+		if (vol_change->m_midi_time > midi_time) return volume;
+		volume = vol_change->m_volume;
+	}
+	return volume;
+}
+
 void NedStaff::prepareReplay() {
 	int i;
+	GList *lptr;
+
+	m_volume_changes = NULL;
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->collectVolumeChanges(&m_volume_changes);
+	}
+
 	for (i = 0; i < VOICE_COUNT; i++) {
 		m_voices[i]->prepareReplay();
 	}
+
+	if (m_volume_changes != NULL) {
+		if ((lptr = g_list_last(m_volume_changes)) != NULL) {
+			getMainWindow()->m_staff_contexts[m_staff_number].m_current_midi_volume =
+				((VolumeChange *) lptr->data)->m_volume;
+		}
+
+		for (lptr = g_list_first(m_volume_changes); lptr; lptr = g_list_next(lptr)) {
+			delete (VolumeChange *) lptr->data;
+		}
+		g_list_free(m_volume_changes);
+		m_volume_changes = NULL;
+	}
 }
 
 void NedStaff::findAccidentals(char offs_array[115], NedMeasure *meas_info, unsigned long long  midi_time) {
@@ -805,6 +962,34 @@ void NedStaff::removeUnneededAccidentals() {
 	}
 }
 
+bool NedStaff::detectVoices(unsigned int *voice_mask, unsigned long long *e_time) {
+	int i;
+	bool not_empty = false, ne;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		ne = m_voices[i]->detectVoices(voice_mask, e_time);
+		not_empty = not_empty || ne;
+	}
+	return not_empty;
+}
+
+void NedStaff::exportLilyPond(FILE *fp, int voice_nr, int *last_line, unsigned int *midi_len,
+		bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map) {
+	if (voice_nr < 0 || voice_nr >= VOICE_COUNT) {
+		NedResource::Abort("NedStaff::exportLilyPond");
+	}
+	m_voices[voice_nr]->exportLilyPond(fp, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map);
+}
+
+void NedStaff::exportLilyLyrics(FILE *fp, bool last_system, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count) {
+	if (voice_nr < 0 || voice_nr >= VOICE_COUNT) {
+		NedResource::Abort("NedStaff::exportLilyLyrics");
+	}
+	m_voices[voice_nr]->exportLilyLyrics(fp, last_system, line_nr, end_time, sil_count);
+}
+	
+
+
 
 void NedStaff::collectElmentsAfter(unsigned long long midi_time, VoiceInfoContainer *containers, int *num_containers) {
 	int i;
