@@ -48,12 +48,15 @@
 
 
 NedSystem::NedSystem(NedPage *page, double ypos, double width, int nr, unsigned int start_measure_number, bool start) :
-m_system_start(CLEF_SPACE+ TIME_SIGNATURE_SPACE), m_isUntouched(TRUE), m_staffs(NULL), m_deleted_staffs(NULL), m_ypos(ypos), m_width(width), m_system_number(nr),  m_page(page), m_measure_count(0), m_only_whole_elements(FALSE) {
+m_system_start(CLEF_SPACE), m_isUntouched(TRUE), m_staffs(NULL), m_deleted_staffs(NULL), m_ypos(ypos), m_width(width), m_system_number(nr),  m_page(page), m_measure_count(0), m_only_whole_elements(FALSE) {
 	int i;
 	double staff_pos = 0;
 	double indent;
 	NedStaff *staff;
 	GList *lptr;
+	if (page->getPageNumber() == 0 &&  nr == 0) {
+		m_system_start += TIME_SIGNATURE_SPACE;
+	}
 	for (i = 0; i < MAX_MEASURES; m_measures[i++].setSystem(this));
 	if (start) {
 		for (i = 0; i < getMainWindow()->getStaffCount(); i++) {
@@ -101,8 +104,6 @@ NedStaff *NedSystem::getStaff(int nr) {
 
 void NedSystem::fill_up(double indent, NedCommandList *command_list) {
 	GList *lptr;
-
-
 	do {
 		for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
 			((NedStaff *) lptr->data)->appendWholeRest(command_list);
@@ -125,6 +126,22 @@ double NedSystem::getHeight() {
 
 NedMainWindow *NedSystem::getMainWindow() {return m_page->getMainWindow();}
 
+/*
+bool NedSystem::find_new_cursor_pos(guint keyval, int *x, int *y, double *dist) {
+	GList *lptr;
+	double newx = x, newy = y;
+	bool found = false;
+
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		if (((NedSystem *) lptr->data)->find_new_cursor_pos(keyval, &newx, &newy, dist)) {
+			found = true;
+			*x = newx; *y = newy;
+		}
+	}
+	return found;
+}
+*/
+
 void NedSystem::draw(cairo_t *cr, bool first_page) {
 	double leftx = getMainWindow()->getLeftX();
 	double topy = getMainWindow()->getTopY();
@@ -262,17 +279,37 @@ void NedSystem::empty() {
 	}
 }
 
-
-bool NedSystem::trySelect(double x, double y) {
+bool NedSystem::hasOnlyRests() {
 	GList *lptr;
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
-		if (((NedStaff *) lptr->data)->trySelect(x, y)) {
+		if (!((NedStaff *) lptr->data)->hasOnlyRests()) return false;
+	}
+	return true;
+}
+
+
+bool NedSystem::trySelect(double x, double y, bool only_free_placeables) {
+	GList *lptr;
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		if (((NedStaff *) lptr->data)->trySelect(x, y, only_free_placeables)) {
 			return TRUE;
 		}
 	}
 	return FALSE;
 }
 
+NedChordOrRest *NedSystem::findNearestElement(int staff_nr, double x, double y, NedStaff *oldstaff, double *mindist, double *ydist) {
+	NedChordOrRest *nearestElement = NULL, *element;
+	GList *lptr;
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		if (staff_nr >= 0 && ((NedStaff *) lptr->data)->getStaffNumber() != staff_nr) continue;
+		if ((element = ((NedStaff *) lptr->data)->findNearestElement(x, y, oldstaff, mindist, ydist)) != NULL) {
+			nearestElement = element;
+		}
+	}
+	return nearestElement;
+}
+
 bool NedSystem::tryChangeLength(NedChordOrRest *chord_or_rest) {
 	GList *lptr;
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
@@ -312,6 +349,7 @@ void NedSystem::removeLastImported() {
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
 		((NedStaff *) lptr->data)->removeLastImported();
 	}
+	m_measure_count--;
 }
 
 bool NedSystem::truncateAtStart(NedCommandList *command_list, unsigned long long midi_time) {
@@ -340,7 +378,7 @@ void NedSystem::prepareForImport() {
 	}
 }
 
-void NedSystem::appendElementsOfMeasureLength(part *parts, unsigned int meas_duration) {
+void NedSystem::appendElementsOfMeasureLength(part *parts, unsigned int meas_duration, int meas_num) {
 	NedStaff *staff;
 	part *part_ptr;
 	int i;
@@ -349,7 +387,7 @@ void NedSystem::appendElementsOfMeasureLength(part *parts, unsigned int meas_dur
 	for (i = 0, lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr), i++) {
 		staff = (NedStaff *) lptr->data;
 		part_ptr = &(parts[i]);
-		staff->appendElementsOfMeasureLength(part_ptr, meas_duration);
+		staff->appendElementsOfMeasureLength(part_ptr, meas_duration, meas_num);
 	}
 	m_isUntouched = FALSE;
 }
@@ -371,7 +409,7 @@ void NedSystem::removeUnneededAccidentals() {
 }
 
 
-bool NedSystem::tryInsertOrErease(double x, double y) {
+bool NedSystem::tryInsertOrErease(double x, double y, NedChordOrRest **newObj, bool force_rest) {
 	GList *lptr;
 	double mindist, d;
 	lptr = g_list_first(m_staffs);
@@ -383,14 +421,14 @@ bool NedSystem::tryInsertOrErease(double x, double y) {
 			mindist = d;
 		}
 	}
-	if (nearest_staff->tryInsertOrErease(x, y)) {
+	if (nearest_staff->tryInsertOrErease(x, y, newObj, force_rest)) {
 		return TRUE;
 	}
 	return FALSE;
 }
 
 
-bool NedSystem::findLine(double x, double y, int *ypos, int *line, int *bottom) {
+bool NedSystem::findLine(double x, double y, double *ypos, int *line, double *bottom, NedStaff **staff) {
 	GList *lptr;
 	double mindist, d;
 	lptr = g_list_first(m_staffs);
@@ -403,6 +441,7 @@ bool NedSystem::findLine(double x, double y, int *ypos, int *line, int *bottom)
 		}
 	}
 	if (nearest_staff->findLine(x, y, ypos, line, bottom)) {
+		*staff = nearest_staff;
 		return TRUE;
 	}
 	return FALSE;
@@ -439,11 +478,31 @@ void NedSystem::appendStaff(NedCommandList *command_list, int p_staff_nr /* = -1
 		staff_nr = p_staff_nr;
 	}
 	staff = new NedStaff(this, 10.0 /* dummy */, m_width, staff_nr, TRUE);
-	staff->appendAppropriateWholes();
+	if (getMainWindow()->getUpBeatInverse() > 0 && m_page->getPageNumber() == 0 && m_system_number == 0) {
+		staff->setUpbeatRests(getMainWindow()->getNumerator() * WHOLE_NOTE / getMainWindow()->getDenominator() - getMainWindow()->getUpBeatInverse());
+		staff->appendAppropriateWholes(1);
+	}
+	else {
+		staff->appendAppropriateWholes(0);
+	}
 
 	command_list->addCommand(new NedAppendStaffCommand(this, staff));
 }
 
+NedChordOrRest *NedSystem::getFirstObjectInNextSystem(int staff_nr) {
+	NedSystem *system;
+	GList *lptr;
+
+	if ((system = m_page->getNextSystem(this)) == NULL) return NULL;
+
+	if ((lptr = g_list_nth(system->m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::getFirstObjectInNextSystem");
+	}
+	lptr = ((NedStaff *) lptr->data)->getFirstChordOrRest(0, 0, false);
+	if (lptr == NULL) return NULL;
+	return (NedChordOrRest *) lptr->data;
+}
+
 void NedSystem::do_append_staff(NedStaff *staff) {
 	m_staffs = g_list_append(m_staffs, staff);
 }
@@ -457,6 +516,22 @@ void NedSystem::do_remove_staff(NedStaff *staff) {
 	m_staffs = g_list_delete_link(m_staffs, lptr);
 }
 
+void NedSystem::cutEmptyVoices() {
+	GList *lptr;
+
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		((NedStaff *) lptr->data)->cutEmptyVoices();
+	}
+}
+
+void NedSystem::testTies() {
+	GList *lptr;
+
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		((NedStaff *) lptr->data)->testTies();
+	}
+}
+
 void NedSystem::removeLastStaff() {
 	GList *lptr;
 	NedStaff *staff;
@@ -537,6 +612,9 @@ bool NedSystem::reposit(int pass, int measure_number, NedCommandList *command_li
 	unsigned int meas_duration;
 	bool use_upbeat;
 	double indent = (use_upbeat = (m_system_number == 0 && m_page->getPageNumber() == 0)) ? getMainWindow()->getFirstSystemIndent() : 0.0;
+	if (getMainWindow()->isTimsigChangingMeasure(getNumberOfFirstMeasure())) {
+		indent = SIG_X_SHIFT;
+	}
 
 	if (next_system != NULL) *next_system = NULL;
 	if (measure_number < 0) {
@@ -756,11 +834,31 @@ void NedSystem::findSelectedSystems(NedBbox *sel_rect, int *number_of_first_sele
 	}
 }
 
-void NedSystem::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array) {
+void NedSystem::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, bool first) {
 	GList *lptr;
-	m_system_start = CLEF_SPACE+ TIME_SIGNATURE_SPACE;
+	m_system_start = CLEF_SPACE;
+	if (first) {
+		m_system_start += TIME_SIGNATURE_SPACE;
+	}
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
-		((NedStaff *) lptr->data)->setAndUpdateClefTypeAndKeySig(clef_and_key_array);
+		((NedStaff *) lptr->data)->setAndUpdateClefTypeAndKeySig(clef_and_key_array, first);
+	}
+}
+
+void NedSystem::determineVolume(unsigned int *vol, NedChordOrRest *element, int staff_nr) {
+	GList *lptr;
+
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::determineVolume");
+	}
+
+	((NedStaff *) lptr->data)->determineVolume(vol, element, staff_nr);
+}
+
+void NedSystem::determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found) {
+	GList *lptr;
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		((NedStaff *) lptr->data)->determineTempoInverse(element, till, tempoinverse, found);
 	}
 }
 
@@ -966,7 +1064,7 @@ void NedSystem::do_reposit(bool use_upbeat, double indent) {
 					current_position +=  m_measures[new_measure_count].getNeededSpaceBefore();
 				}
 				if (new_measure_count >= MAX_MEASURES) {
-					printf("sys = %d, position_counter = %d, midi_time = %llu, new_measure_count = %d\n",
+					NedResource::DbgMsg(DBG_CRITICAL, "sys = %d, position_counter = %d, midi_time = %llu, new_measure_count = %d\n",
 					m_system_number, position_counter, midi_time, new_measure_count);
 					NedResource::Abort("NedSystem::do_reposit: new_measure_count >= MAX_MEASURES");
 				}
@@ -1192,7 +1290,7 @@ void NedSystem::compute_extra_space(double indent) {
 					current_position +=  m_measures[new_measure_count].getNeededSpaceBefore();
 				}
 				if (new_measure_count >= MAX_MEASURES) {
-					printf("sys = %d, position_counter = %d, midi_time = %llu, new_measure_count = %d\n",
+					NedResource::DbgMsg(DBG_CRITICAL, "sys = %d, position_counter = %d, midi_time = %llu, new_measure_count = %d\n",
 					m_system_number, position_counter, midi_time, new_measure_count);
 					NedResource::Abort("NedSystem::compute_extra_space: new_measure_count >= MAX_MEASURES");
 				}
@@ -1324,12 +1422,12 @@ NedMeasure *NedSystem::getMeasure(unsigned int midi_time) {
 		}
 	}
 #ifdef AAA
-	printf("pag: %d, sys. %d, midi_time = %u , m_measures[m_measure_count(%d)].midi_start = %llu, m_measures[m_measure_count(%d)].midi_end = %llu, NOTE_4 = %u\n",
+	NedResource::DbgMsg(DBG_TESTING, "pag: %d, sys. %d, midi_time = %u , m_measures[m_measure_count(%d)].midi_start = %llu, m_measures[m_measure_count(%d)].midi_end = %llu, NOTE_4 = %u\n",
 	m_page->getPageNumber(), m_system_number,
-	midi_time, m_measure_count, m_measures[m_measure_count].midi_start, m_measure_count, m_measures[m_measure_count].midi_end, NOTE_4); fflush(stdout);
+	midi_time, m_measure_count, m_measures[m_measure_count].midi_start, m_measure_count, m_measures[m_measure_count].midi_end, NOTE_4); 
 	if (m_measure_count >= 0) {
-		printf("m_measures[m_measure_count - 1(%d)].midi_start = %llu,  m_measures[m_measure_count - 1(%d)].midi_end = %llu\n",
-		m_measure_count-1, m_measures[m_measure_count-1].midi_start, m_measure_count-1, m_measures[m_measure_count-1].midi_end); fflush(stdout);
+		NedResource::DbgMsg(DBG_TESTING, "m_measures[m_measure_count - 1(%d)].midi_start = %llu,  m_measures[m_measure_count - 1(%d)].midi_end = %llu\n",
+		m_measure_count-1, m_measures[m_measure_count-1].midi_start, m_measure_count-1, m_measures[m_measure_count-1].midi_end); 
 	}
 	NedResource::Abort("NedSystem::getMeasure");
 #endif
@@ -1345,6 +1443,7 @@ void NedSystem::prepareReplay() {
 	GList *lptr;
 
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		if (getMainWindow()->m_staff_contexts[((NedStaff *) lptr->data)->getStaffNumber()].m_muted) continue;
 		((NedStaff *) lptr->data)->prepareReplay();
 	}
 }
@@ -1434,11 +1533,56 @@ void NedSystem::restoreSystem(FILE *fp) {
 		}
 		staff = new NedStaff(this, staff_pos, m_width, i, FALSE);
 
-		staff->restoreStaff(fp);
 		m_staffs = g_list_append(m_staffs, staff);
+		staff->restoreStaff(fp);
 
 		staff_pos += staff->getHeight() + DEFAULT_SYSTEM_DIST;
 	}
 }
 
+void NedSystem::detectVoices(int staff_nr, unsigned int *voice_mask, NedSystem **last_system, unsigned long long *end_time) {
+	GList *lptr;
+	unsigned long long e_time = 0;
 
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::detectVoices");
+	}
+	if (((NedStaff *) lptr->data)->detectVoices(voice_mask, &e_time)) {
+		if (*last_system == NULL) {
+			*last_system = this;
+			*end_time = e_time;
+		}
+		else if ((*last_system)->m_page->getPageNumber() < m_page->getPageNumber()) {
+			*last_system = this;
+			*end_time = e_time;
+		}
+		else if ((*last_system)->m_system_number < m_system_number) {
+			*last_system = this;
+			*end_time = e_time;
+		}
+		else if (*last_system == this) {
+			if (e_time > *end_time) {
+				*end_time = e_time;
+			}
+		}
+	}
+}
+
+void NedSystem::exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
+		NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map) {
+	GList *lptr;
+
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::exportLilyPond");
+	}
+	((NedStaff *) lptr->data)->exportLilyPond(fp, voice_nr, last_line, midi_len, last_system == this, end_time, in_alternative, lily_slur, lyrics_map);
+}
+
+void NedSystem::exportLilyLyrics(FILE *fp, bool last_system, int staff_nr, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count) {
+	GList *lptr;
+
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::exportLilyLyrics");
+	}
+	((NedStaff *) lptr->data)->exportLilyLyrics(fp, last_system, voice_nr, line_nr, end_time, sil_count);
+}
